# Copyright [2015] Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from troveclient import client as base_client
from troveclient.tests import utils
from troveclient.v1 import client
from urllib import parse


def get_version_map():
    return {
        '1.0': 'troveclient.tests.fakes.FakeClient',
    }


def assert_has_keys(dict, required=None, optional=None):
    required = required or []
    optional = optional or []
    keys = dict.keys()
    for k in required:
        try:
            assert k in keys
        except AssertionError:
            raise AssertionError("key: %s not found." % k)


class FakeClient(client.Client):

    URL_QUERY_SEPARATOR = '&'
    URL_SEPARATOR = '?'

    def __init__(self, *args, **kwargs):
        client.Client.__init__(self, 'username', 'password',
                               'project_id', 'auth_url',
                               extensions=kwargs.get('extensions'))
        self.client = FakeHTTPClient(**kwargs)

    def _order_url_query_str(self, url):
        """Returns the url with the query strings ordered, if they exist and
        there's more than one. Otherwise the url is returned unaltered.
        """
        if self.URL_QUERY_SEPARATOR in url:
            parts = url.split(self.URL_SEPARATOR)
            if len(parts) == 2:
                queries = sorted(parts[1].split(self.URL_QUERY_SEPARATOR))
                url = self.URL_SEPARATOR.join(
                    [parts[0], self.URL_QUERY_SEPARATOR.join(queries)])
        return url

    def assert_called(self, method, url, body=None, pos=-1):
        """Assert than an API method was just called."""
        expected = (method, utils.order_url(url))
        called = (self.client.callstack[pos][0],
                  utils.order_url(self.client.callstack[pos][1]))

        assert self.client.callstack, \
            "Expected %s %s but no calls were made." % expected

        assert expected == called, \
            'Expected %s %s; got %s %s' % (expected + called)

        if body is not None:
            if self.client.callstack[pos][2] != body:
                raise AssertionError('%r != %r' %
                                     (self.client.callstack[pos][2], body))

    def assert_called_anytime(self, method, url, body=None):
        """Assert than an API method was called anytime in the test."""
        expected = (method, utils.order_url(url))

        assert self.client.callstack, \
            "Expected %s %s but no calls were made." % expected

        found = False
        for entry in self.client.callstack:
            if expected == (entry[0], utils.order_url(entry[1])):
                found = True
                break

        assert found, 'Expected %s; got %s' % (expected, self.client.callstack)
        if body is not None:
            try:
                assert entry[2] == body
            except AssertionError:
                print(entry[2])
                print("!=")
                print(body)
                raise

        self.client.callstack = []


class FakeHTTPClient(base_client.HTTPClient):

    def __init__(self, **kwargs):
        self.username = 'username'
        self.password = 'password'
        self.auth_url = 'auth_url'
        self.management_url = (
            'http://trove-api:8779/v1.0/14630bc0e9ef4e248c9753eaf57b0f6e')
        self.tenant_id = 'tenant_id'
        self.callstack = []
        self.projectid = 'projectid'
        self.user = 'user'
        self.region_name = 'region_name'
        self.endpoint_type = 'endpoint_type'
        self.service_type = 'service_type'
        self.service_name = 'service_name'
        self.volume_service_name = 'volume_service_name'
        self.timings = 'timings'
        self.bypass_url = 'bypass_url'
        self.os_cache = 'os_cache'
        self.http_log_debug = 'http_log_debug'

    def _cs_request(self, url, method, **kwargs):
        # Check that certain things are called correctly
        if method in ['GET', 'DELETE']:
            assert 'body' not in kwargs
        elif method == 'PUT':
            assert 'body' in kwargs

        if url is not None:
            # Call the method
            args = parse.parse_qsl(parse.urlparse(url)[4])
            kwargs.update(args)
            munged_url = url.rsplit('?', 1)[0]
            munged_url = munged_url.strip('/').replace('/', '_')
            munged_url = munged_url.replace('.', '_')
            munged_url = munged_url.replace('-', '_')
            munged_url = munged_url.replace(' ', '_')
            callback = "%s_%s" % (method.lower(), munged_url)

        if not hasattr(self, callback):
            raise AssertionError('Called unknown API method: %s %s, '
                                 'expected fakes method name: %s' %
                                 (method, url, callback))

        # Note the call
        self.callstack.append((method, url, kwargs.get('body')))

        status, headers, body = getattr(self, callback)(**kwargs)
        r = utils.TestResponse({
            "status_code": status,
            "text": body,
            "headers": headers,
        })
        return r, body

    def get_instances(self, **kw):
        return (
            200,
            {},
            {
                "instances": [
                    {
                        "id": "1234",
                        "name": "test-member-1",
                        "status": "ACTIVE",
                        "addresses": [
                            {"type": "private", "address": "10.0.0.13"}
                        ],
                        "volume": {"size": 2},
                        "flavor": {"id": "02"},
                        "region": "regionOne",
                        "datastore": {
                            "version": "5.6", "type": "mysql",
                            "version_number": "5.7.29"
                        },
                        "tenant_id": "fake_tenant_id",
                        "replica_of": {"id": "fake_master_id"},
                        "access": {"is_public": False, "allowed_cidrs": []}
                    },
                    {
                        "id": "5678",
                        "name": "test-member-2",
                        "status": "ACTIVE",
                        "addresses": [
                            {"type": "private", "address": "10.0.0.14"}
                        ],
                        "volume": {"size": 2},
                        "flavor": {"id": "2"},
                        "region": "regionOne",
                        "datastore": {
                            "version": "5.6", "type": "mysql",
                            "version_number": "5.7.29"
                        },
                        "tenant_id": "fake_tenant_id",
                        "access": {"is_public": False, "allowed_cidrs": []}
                    }
                ]
            }
        )

    def get_instance_counts(self, **kw):
        return (200, {}, {"instances": [
            {
                "module_id": "4321",
                "module_name": "mod1",
                "min_date": "2015-05-02T11:06:16",
                "max_date": "2015-05-02T11:06:19",
                "module_md5": "9db783b92a9355f70c41806659fcb77d",
                "current": True,
                "count": 1}]})

    def get_instances_1234(self, **kw):
        r = {'instance': self.get_instances()[2]['instances'][0]}
        return (200, {}, r)

    def get_instance_create(self, **kw):
        return (200, {}, {"instance": {
                "status": "BUILD",
                "updated": "2017-12-22T20:02:32",
                "name": "test",
                "created": "2017-12-22T20:02:32",
                "networks": {
                    "name": "test-net",
                    "id": "net-id"
                },
                "id": "2468",
                "volume": {
                    "size": 1
                },
                "flavor": {
                    "id": "310"
                },
                "datastore": {
                    "version": "5.6",
                    "type": "mysql",
                    "version_number": "5.7.29"
                }}})

    def post_instances(self, body, **kw):
        assert_has_keys(
            body['instance'],
            required=['name', 'flavorRef'],
            optional=['volume'])
        if 'volume' in body['instance']:
            assert_has_keys(body['instance']['volume'], required=['size'])
        return (202, {}, self.get_instances_1234()[2])

    def get_flavors(self, **kw):
        return (200, {}, {"flavors": [
            {
                "str_id": "1",
                "ram": 512,
                "id": 1,
                "name": "m1.tiny"},
            {
                "str_id": "10",
                "ram": 768,
                "id": 10,
                "name": "eph.rd-smaller"},
            {
                "str_id": "2",
                "ram": 2048,
                "id": 2,
                "name": "m1.small"},
            {
                "str_id": "3",
                "ram": 4096,
                "id": 3,
                "name": "m1.medium"},
            {
                "str_id": "7d0d16e5-875f-4198-b6da-90ab2d3e899e",
                "ram": 8192,
                "id": None,
                "name": "m1.uuid"},
            {
                "str_id": "02",
                "ram": 1024,
                "id": None,
                "name": "m1.leading-zero"}]})

    def get_datastores_mysql_versions_some_version_id_flavors(self, **kw):
        return self.get_flavors()

    def get_flavors_1(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][0]}
        return (200, {}, r)

    def get_flavors_2(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][2]}
        return (200, {}, r)

    def get_flavors_m1_tiny(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][0]}
        return (200, {}, r)

    def get_flavors_eph_rd_smaller(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][1]}
        return (200, {}, r)

    def get_flavors_m1_small(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][2]}
        return (200, {}, r)

    def get_flavors_m1_medium(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][3]}
        return (200, {}, r)

    def get_flavors_m1_uuid(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][4]}
        return (200, {}, r)

    def get_flavors_02(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][5]}
        return (200, {}, r)

    def get_flavors_m1_leading_zero(self, **kw):
        r = {'flavor': self.get_flavors()[2]['flavors'][5]}
        return (200, {}, r)

    def get_volume_types(self, **kw):
        return (200, {}, {"volume_types": [
            {
                "id": "1",
                "name": "vt_1",
                "description": "Volume type #1",
                "is_public": False},
            {
                "id": "10",
                "name": "volume_type_2",
                "description": "Volume type #2",
                "is_public": True}]})

    def get_datastores_mysql_versions_some_version_id_volume_types(self, **kw):
        return self.get_volume_types()

    def get_volume_types_1(self, **kw):
        r = {'volume_type': self.get_volume_types()[2]['volume_types'][0]}
        return (200, {}, r)

    def get_volume_types_2(self, **kw):
        r = {'volume_type': self.get_volume_types()[2]['volume_types'][2]}
        return (200, {}, r)

    def get_clusters(self, **kw):
        return (200, {}, {"clusters": [
            {
                "instances": [
                    {
                        "type": "member",
                        "id": "member-1",
                        "ip": ["10.0.0.3"],
                        "flavor": {"id": "02"},
                        "name": "test-clstr-member-1"
                    },
                    {
                        "type": "member",
                        "id": "member-2",
                        "ip": ["10.0.0.4"],
                        "flavor": {"id": "2"},
                        "name": "test-clstr-member-2"
                    }],
                "updated": "2015-05-02T11:06:19",
                "task": {"description": "No tasks for the cluster.", "id": 1,
                         "name": "NONE"},
                "name": "test-clstr",
                "created": "2015-05-02T10:37:04",
                "datastore": {"version": "7.1", "type": "vertica"},
                "id": "cls-1234"}]})

    def get_clusters_cls_1234(self, **kw):
        # NOTE(zhaochao): getting a cluster will load instances
        # informations while getting the list of clusters won't,
        # so we cannot just reuse the reponse of getting clusters
        # for getting a cluster.
        # The following response body is almost identical to the
        # result of get_clusters, except the additional 'status'
        # and 'volume' items, so all existing unitests accessing
        # this piece of fake response will continue to work
        # without any modification.
        return (200, {}, {'cluster': {
            "instances": [
                {
                    "type": "member",
                    "id": "member-1",
                    "ip": ["10.0.0.3"],
                    "flavor": {"id": "02"},
                    "name": "test-clstr-member-1",
                    "status": "ACTIVE",
                    "volume": {"size": 2}
                },
                {
                    "type": "member",
                    "id": "member-2",
                    "ip": ["10.0.0.4"],
                    "flavor": {"id": "2"},
                    "name": "test-clstr-member-2",
                    "status": "ACTIVE",
                    "volume": {"size": 2}
                }],
            "updated": "2015-05-02T11:06:19",
            "task": {"description": "No tasks for the cluster.", "id": 1,
                     "name": "NONE"},
            "name": "test-clstr",
            "created": "2015-05-02T10:37:04",
            "datastore": {"version": "7.1", "type": "vertica"},
            "id": "cls-1234"}})

    def get_cluster_instance_modules(self, **kw):
        return (200, {}, {
            "modules": [
                {
                    "auto_apply": False,
                    "contents": None,
                    "created": "2018-04-17 05:34:02.84",
                    "datastore": "mariadb",
                    "datastore_version": "all",
                    "id": "module-1",
                    "is_admin": False,
                    "md5": "md5-1",
                    "message": "Module.V1",
                    "name": "mymod1",
                    "removed": None,
                    "status": "OK",
                    "tenant": "7f1f041fc291455b83a0b3eb98140808",
                    "type": "ping",
                    "updated": "2018-04-17 05:34:02.84",
                    "visible": True},

                {
                    "auto_apply": False,
                    "contents": None,
                    "created": "2018-04-17 05:34:02.84",
                    "datastore": "mariadb",
                    "datastore_version": "all",
                    "id": "module-2",
                    "is_admin": False,
                    "md5": "md5-2",
                    "message": "Module.V1",
                    "name": "mymod2",
                    "removed": None,
                    "status": "OK",
                    "tenant": "7f1f041fc291455b83a0b3eb98140808",
                    "type": "ping",
                    "updated": "2018-04-17 05:34:02.84",
                    "visible": True}]})

    def delete_instances_1234(self, **kw):
        return (202, {}, None)

    def delete_clusters_cls_1234(self, **kw):
        return (202, {}, None)

    def patch_instances_1234(self, **kw):
        return (202, {}, None)

    def post_clusters(self, body, **kw):
        assert_has_keys(
            body['cluster'],
            required=['instances', 'datastore', 'name'])
        if 'instances' in body['cluster']:
            for instance in body['cluster']['instances']:
                assert_has_keys(instance, required=['volume', 'flavorRef'])
        return (202, {}, self.get_clusters_cls_1234()[2])

    def post_clusters_cls_1234(self, body, **kw):
        return (202, {}, None)

    def post_instances_1234_action(self, **kw):
        return (202, {}, None)

    def get_datastores(self, **kw):
        return (200, {}, {"datastores": [
            {
                "default_version": "v-56",
                "versions": [{"id": "v-56", "name": "5.6"}],
                "id": "d-123",
                "name": "mysql"},
            {
                "default_version": "v-71",
                "versions": [{"id": "v-71", "name": "7.1"}],
                "id": "d-456",
                "name": "vertica"
            }]})

    def get_datastores_d_123(self, **kw):
        r = {'datastore': self.get_datastores()[2]['datastores'][0]}
        return (200, {}, r)

    def get_datastores_d_123_versions(self, **kw):
        return (200, {}, {"versions": [
            {
                "datastore": "d-123",
                "id": "v-56",
                "name": "5.6"}]})

    def get_datastores_d_123_versions_v_56(self, **kw):
        r = {'version': self.get_datastores_d_123_versions()[2]['versions'][0]}
        return (200, {}, r)

    def get_configurations(self, **kw):
        return (200, {}, {"configurations": [
            {
                "datastore_name": "mysql",
                "updated": "2015-05-16T10:24:29",
                "name": "test_config",
                "created": "2015-05-16T10:24:28",
                "datastore_version_name": "5.6",
                "datastore_version_number": "5.7.29",
                "id": "c-123",
                "values": {"max_connections": 5},
                "datastore_version_id": "d-123", "description": ''}]})

    def get_configurations_c_123(self, **kw):
        r = {'configuration': self.get_configurations()[2]['configurations'][0]
             }
        return (200, {}, r)

    def get_datastores_d_123_versions_v_156_parameters(self, **kw):
        return (200, {}, {"configuration-parameters": [
            {
                "type": "string",
                "name": "character_set_results",
                "datastore_version_id": "d-123",
                "restart_required": "false"},
            {
                "name": "connect_timeout",
                "min": 2,
                "max": 31536000,
                "restart_required": "false",
                "type": "integer",
                "datastore_version_id": "d-123"},
            {
                "type": "string",
                "name": "character_set_client",
                "datastore_version_id": "d-123",
                "restart_required": "false"},
            {
                "name": "max_connections",
                "min": 1,
                "max": 100000,
                "restart_required": "false",
                "type": "integer",
                "datastore_version_id": "d-123"}]})

    def get_datastores_d_123_versions_v_56_parameters_max_connections(self,
                                                                      **kw):
        r = self.get_datastores_d_123_versions_v_156_parameters()[
            2]['configuration-parameters'][3]
        return (200, {}, r)

    def get_configurations_c_123_instances(self, **kw):
        return (200, {}, {"instances": [
            {
                "id": "1",
                "name": "instance-1"},
            {
                "id": "2",
                "name": "instance-2"}]})

    def delete_configurations_c_123(self, **kw):
        return (202, {}, None)

    def get_instances_1234_configuration(self, **kw):
        return (200, {}, {"instance": {"configuration": {
                "tmp_table_size": "15M",
                "innodb_log_files_in_group": "2",
                "skip-external-locking": "1",
                "max_user_connections": "98"}}})

    def put_instances_1234(self, **kw):
        return (202, {}, None)

    def patch_instances_1234_metadata_key_123(self, **kw):
        return (202, {}, None)

    def put_instances_1234_metadata_key_123(self, **kw):
        return (202, {}, None)

    def delete_instances_1234_metadata_key_123(self, **kw):
        return (202, {}, None)

    def post_instances_1234_metadata_key123(self, body, **kw):
        return (202, {}, {'metadata': {}})

    def get_instances_1234_metadata(self, **kw):
        return (200, {}, {"metadata": {}})

    def get_instances_1234_metadata_key123(self, **kw):
        return (200, {}, {"metadata": {}})

    def get_modules(self, **kw):
        return (200, {}, {"modules": [
            {
                "id": "4321",
                "name": "mod1",
                "type": "ping",
                "datastore": 'all',
                "datastore_version": 'all',
                "tenant": 'all',
                "auto_apply": 0,
                "visible": 1,
                "priority_apply": 0,
                "apply_order": 5,
                "is_admin": 0},
            {
                "id": "8765",
                "name": "mod2",
                "type": "ping",
                "datastore": 'all',
                "datastore_version": 'all',
                "tenant": 'all',
                "auto_apply": 0,
                "visible": 0,
                "priority_apply": 0,
                "apply_order": 5,
                "is_admin": 1}]})

    def get_modules_4321(self, **kw):
        r = {'module': self.get_modules()[2]['modules'][0]}
        return (200, {}, r)

    def get_modules_8765(self, **kw):
        r = {'module': self.get_modules()[2]['modules'][1]}
        return (200, {}, r)

    def post_modules(self, **kw):
        r = {'module': self.get_modules()[2]['modules'][0]}
        return (200, {}, r)

    def put_modules_4321(self, **kw):
        return (200, {}, {"module": {'name': 'mod3'}})

    def delete_modules_4321(self, **kw):
        return (200, {}, None)

    def get_instances_1234_modules(self, **kw):
        return (200, {}, {"modules": [{"module": {}}]})

    def get_modules_4321_instances(self, **kw):
        if kw.get('count_only', False):
            return self.get_instance_counts()
        return self.get_instances()

    def put_modules_4321_instances(self, **kw):
        return (202, {}, None)

    def get_instances_modules(self, **kw):
        return (200, {}, None)

    def get_instances_member_1_modules(self, **kw):
        return self.get_modules()

    def get_instances_member_2_modules(self, **kw):
        return self.get_modules()

    def post_instances_1234_modules(self, **kw):
        r = {'modules': [self.get_modules()[2]['modules'][0]]}
        return (200, {}, r)

    def delete_instances_1234_modules_4321(self, **kw):
        return (200, {}, None)

    def get_limits(self, **kw):
        return (200, {}, {"limits": [
            {
                "max_backups": 50,
                "verb": "ABSOLUTE",
                "max_volumes": 20,
                "max_instances": 5}]})

    def get_backups(self, **kw):
        return (200, {}, {"backups": [
            {
                "status": "COMPLETED",
                "updated": "2015-05-16T14:23:08",
                "description": None,
                "datastore": {"version": "5.6", "type": "mysql",
                              "version_id": "v-56"},
                "id": "bk-1234",
                "size": 0.11,
                "name": "bkp_1",
                "created": "2015-05-16T14:22:28",
                "instance_id": "1234",
                "parent_id": None,
                "locationRef": ("http://backup_srvr/database_backups/"
                                "bk-1234.xbstream.gz.enc"),
                "project_id": "262db161-d3e4-4218-8bde-5bd879fc3e61"
            },
            {
                "status": "COMPLETED",
                "updated": "2015-05-16T14:22:12",
                "description": None,
                "datastore": {"version": "5.6", "type": "mysql",
                              "version_id": "v-56"},
                "id": "bk-5678",
                "size": 0.11,
                "name": "test_bkp",
                "created": "2015-05-16T14:21:27",
                "instance_id": "5678",
                "parent_id": None,
                "locationRef": ("http://backup_srvr/database_backups/"
                                "bk-5678.xbstream.gz.enc"),
                "project_id": "262db161-d3e4-4218-8bde-5bd879fc3e61"
            }
        ]})

    def get_backups_bk_1234(self, **kw):
        r = {'backup': self.get_backups()[2]['backups'][0]}
        return (200, {}, r)

    def get_backups_bkp_1(self, **kw):
        r = {'backup': self.get_backups()[2]['backups'][0]}
        return (200, {}, r)

    def get_instances_1234_backups(self, **kw):
        r = {'backups': [self.get_backups()[2]['backups'][0]]}
        return (200, {}, r)

    def delete_backups_bk_1234(self, **kw):
        return (202, {}, None)

    def post_backups(self, body, **kw):
        assert_has_keys(
            body['backup'],
            required=['name'],
            optional=['description', 'parent'])
        return (202, {}, self.get_backups_bk_1234()[2])

    def get_instances_1234_databases(self, **kw):
        return (200, {}, {"databases": [
            {"name": "db_1"},
            {"name": "db_2"},
            {"name": "performance_schema"}]})

    def delete_instances_1234_databases_db_1(self, **kw):
        return (202, {}, None)

    def post_instances_1234_databases(self, body, **kw):
        assert_has_keys(
            body,
            required=['databases'])
        for database in body['databases']:
            assert_has_keys(database, required=['name'],
                            optional=['character_set', 'collate'])
        return (202, {},
                self.get_instances_1234_databases()[2]['databases'][0])

    def get_instances_1234_users(self, **kw):
        return (200, {}, {"users": [
            {"host": "%", "name": "jacob", "databases": []},
            {"host": "%", "name": "rocky", "databases": []},
            {"host": "%", "name": "harry", "databases": [{"name": "db1"}]}]})

    def get_instances_1234_users_jacob(self, **kw):
        r = {'user': self.get_instances_1234_users()[2]['users'][0]}
        return (200, {}, r)

    def delete_instances_1234_users_jacob(self, **kw):
        return (202, {}, None)

    def post_instances_1234_users(self, body, **kw):
        assert_has_keys(
            body,
            required=['users'])
        for database in body['users']:
            assert_has_keys(database, required=['name', 'password'],
                            optional=['databases'])
        return (202, {}, self.get_instances_1234_users()[2]['users'][0])

    def get_instances_1234_users_jacob_databases(self, **kw):
        r = {'databases': [
            self.get_instances_1234_databases()[2]['databases'][0],
            self.get_instances_1234_databases()[2]['databases'][1]]}
        return (200, {}, r)

    def put_instances_1234_users_jacob(self, **kw):
        return (202, {}, None)

    def put_instances_1234_users_jacob_databases(self, **kw):
        return (202, {}, None)

    def delete_instances_1234_users_jacob_databases_db1(self, **kw):
        return (202, {}, None)

    def post_instances_1234_root(self, **kw):
        return (202, {}, {"user": {"password": "password", "name": "root"}})

    def post_clusters_cls_1234_root(self, **kw):
        return (202, {}, {"user": {"password": "password", "name": "root"}})

    def delete_instances_1234_root(self, **kw):
        return (202, {}, None)

    def get_instances_1234_root(self, **kw):
        return (200, {}, {"rootEnabled": 'True'})

    def get_instances_master_1(self, **kw):
        return (200, {}, {"instance": {"id": 'myid'}})

    def get_clusters_cls_1234_root(self, **kw):
        return (200, {}, {"rootEnabled": 'True'})

    def get_security_groups(self, **kw):
        return (200, {}, {"security_groups": [
            {
                "instance_id": "1234",
                "updated": "2015-05-16T17:29:45",
                "name": "SecGroup_1234",
                "created": "2015-05-16T17:29:45",
                "rules": [{"to_port": 3306, "cidr": "0.0.0.0/0",
                           "from_port": 3306,
                           "protocol": "tcp", "id": "1"}],
                "id": "2",
                "description": "Security Group for 1234"}]})

    def get_security_groups_2(self, **kw):
        r = {'security_group': self.get_security_groups()[
            2]['security_groups'][0]}
        return (200, {}, r)

    def delete_security_group_rules_2(self, **kw):
        return (202, {}, None)

    def post_security_group_rules(self, body, **kw):
        assert_has_keys(body['security_group_rule'], required=['cidr', 'cidr'])
        return (202, {}, {"security_group_rule": [
            {
                "from_port": 3306,
                "protocol": "tcp",
                "created": "2015-05-16T17:55:05",
                "to_port": 3306,
                "security_group_id": "2",
                "cidr": "15.0.0.0/24", "id": 3}]})

    def get_quotas(self, **kw):
        return (200, {}, {"quotas": [
            {
                "reserved": 1,
                "resource": "instances",
                "limit": 10,
                "in_use": 2
            },
            {
                "reserved": 3,
                "resource": "backups",
                "limit": 50,
                "in_use": 4
            },
            {
                "reserved": 5,
                "resource": "volumes",
                "limit": 40,
                "in_use": 6
            }]})

    def update_instances_quota(self, **kw):
        return (200, {}, {"quotas": {"instances": 51}})

    def get_logs(self, **kw):
        return (200, {}, {"logs": [
            {
                "name": "general",
                "type": "USER",
                "status": "Partial",
                "published": "128",
                "pending": "4096",
                "container": "data_logs",
                "prefix": "mysql-general",
                "metafile": "mysql-general_metafile"
            },
            {
                "name": "slow_query",
                "type": "USER",
                "status": "Ready",
                "published": "0",
                "pending": "128",
                "container": "None",
                "prefix": "None",
                "metafile": "mysql-slow_query_metafile"}]})



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid

from mistralclient.api.client import client as mistral_client

from troveclient import base
from troveclient import common


class Backup(base.Resource):
    """Backup is a resource used to hold backup information."""
    def __repr__(self):
        return "<Backup: %s>" % self.name


class Schedule(base.Resource):
    """Schedule is a resource used to hold information about scheduled backups.
    """
    def __repr__(self):
        return "<Schedule: %s>" % self.name


class ScheduleExecution(base.Resource):
    """ScheduleExecution is a resource used to hold information about
    the execution of a scheduled backup.
    """
    def __repr__(self):
        return "<Execution: %s>" % self.name


class Backups(base.ManagerWithFind):
    """Manage :class:`Backups` information."""

    resource_class = Backup

    def get(self, backup):
        """Get a specific backup.

        :rtype: :class:`Backups`
        """
        return self._get("/backups/%s" % base.getid(backup),
                         "backup")

    def list(self, limit=None, marker=None, datastore=None, instance_id=None,
             all_projects=False, project_id=None):
        """Get a list of all backups."""
        query_strings = {}
        if datastore:
            query_strings["datastore"] = datastore
        if instance_id:
            query_strings["instance_id"] = instance_id
        if all_projects:
            query_strings["all_projects"] = True
        if project_id:
            query_strings["project_id"] = project_id

        return self._paginated("/backups", "backups", limit, marker,
                               query_strings)

    def create(self, name, instance, description=None,
               parent_id=None, incremental=False, swift_container=None,
               restore_from=None, restore_ds_version=None, restore_size=None):
        """Create or restore a new backup.

        :param name: name for backup.
        :param instance: instance to backup.
        :param description: (optional).
        :param parent_id: base for incremental backup (optional).
        :param incremental: flag to indicate incremental backup based on
                            last backup
        :param swift_container: Swift container name.
        :param restore_from: The original backup data location, typically this
                             is a Swift object URL.
        :param restore_ds_version: ID of the local datastore version
                                   corresponding to the original backup.
        :param restore_size: The original backup size.
        :returns: :class:`Backups`
        """
        body = {
            "backup": {
                "name": name,
            }
        }

        if restore_from:
            body['backup'].update({
                'restore_from': {
                    'remote_location': restore_from,
                    'local_datastore_version_id': restore_ds_version,
                    'size': restore_size
                }
            })
        else:
            body['backup']['incremental'] = int(incremental)
            if instance:
                body['backup']['instance'] = base.getid(instance)
            if description:
                body['backup']['description'] = description
            if parent_id:
                body['backup']['parent_id'] = parent_id
            if swift_container:
                body['backup']['swift_container'] = swift_container

        return self._create("/backups", body, "backup")

    def delete(self, backup):
        """Delete the specified backup.

        :param backup: The backup to delete
        """
        url = "/backups/%s" % base.getid(backup)
        resp, body = self.api.client.delete(url)
        common.check_for_exceptions(resp, body, url)

    backup_create_workflow = "trove.backup_create"

    def _get_mistral_client(self):
        if hasattr(self.api.client, 'auth'):
            auth_url = self.api.client.auth.auth_url
            user = self.api.client.auth._username
            key = self.api.client.auth._password
            tenant_name = self.api.client.auth._project_name
        else:
            auth_url = self.api.client.auth_url
            user = self.api.client.user
            key = self.api.client.password
            tenant_name = self.api.client.projectid

        return mistral_client(auth_url=auth_url, username=user, api_key=key,
                              project_name=tenant_name)

    def _build_schedule(self, cron_trigger, wf_input):
        if isinstance(wf_input, str):
            wf_input = json.loads(wf_input)
        sched_info = {"id": cron_trigger.name,
                      "name": wf_input["name"],
                      "instance": wf_input['instance'],
                      "parent_id": wf_input.get('parent_id', None),
                      "created_at": cron_trigger.created_at,
                      "next_execution_time": cron_trigger.next_execution_time,
                      "pattern": cron_trigger.pattern,
                      "input": cron_trigger.workflow_input
                      }
        if hasattr(cron_trigger, 'updated_at'):
            sched_info["updated_at"] = cron_trigger.updated_at
        return Schedule(self, sched_info, loaded=True)

    def schedule_create(self, instance, pattern, name,
                        description=None, incremental=None,
                        mistral_client=None):
        """Create a new schedule to backup the given instance.

        :param instance: instance to backup.
        :param: pattern: cron pattern for schedule.
        :param name: name for backup.
        :param description: (optional).
        :param incremental: flag for incremental backup (optional).
        :returns: :class:`Backups`
        """

        if not mistral_client:
            mistral_client = self._get_mistral_client()

        inst_id = base.getid(instance)
        cron_name = str(uuid.uuid4())
        wf_input = {"instance": inst_id,
                    "name": name,
                    "description": description,
                    "incremental": incremental
                    }

        cron_trigger = mistral_client.cron_triggers.create(
            cron_name, self.backup_create_workflow, pattern=pattern,
            workflow_input=wf_input)

        return self._build_schedule(cron_trigger, wf_input)

    def schedule_list(self, instance, mistral_client=None):
        """Get a list of all backup schedules for an instance.

        :param: instance for which to list schedules.
        :rtype: list of :class:`Schedule`.
        """
        inst_id = base.getid(instance)
        if not mistral_client:
            mistral_client = self._get_mistral_client()

        return [self._build_schedule(cron_trig, cron_trig.workflow_input)
                for cron_trig in mistral_client.cron_triggers.list()
                if inst_id in cron_trig.workflow_input]

    def schedule_show(self, schedule, mistral_client=None):
        """Get details of a backup schedule.

        :param: schedule to show.
        :rtype: :class:`Schedule`.
        """
        if isinstance(schedule, Schedule):
            schedule = schedule.id

        if not mistral_client:
            mistral_client = self._get_mistral_client()

        schedule = mistral_client.cron_triggers.get(schedule)
        return self._build_schedule(schedule, schedule.workflow_input)

    def schedule_delete(self, schedule, mistral_client=None):
        """Remove a given backup schedule.

        :param schedule: schedule to delete.
        """

        if isinstance(schedule, Schedule):
            schedule = schedule.id

        if not mistral_client:
            mistral_client = self._get_mistral_client()

        mistral_client.cron_triggers.delete(schedule)

    def execution_list(self, schedule, mistral_client=None,
                       marker='', limit=None):
        """Get a list of all executions of a scheduled backup.

        :param: schedule for which to list executions.
        :rtype: list of :class:`ScheduleExecution`.
        """

        if isinstance(schedule, Schedule):
            schedule = schedule.id

        if isinstance(marker, ScheduleExecution):
            marker = getattr(marker, 'id')

        if not mistral_client:
            mistral_client = self._get_mistral_client()

        cron_trigger = mistral_client.cron_triggers.get(schedule)
        ct_input = json.loads(cron_trigger.workflow_input)

        def mistral_execution_generator():
            m = marker
            while True:
                try:
                    the_list = mistral_client.executions.list(
                        marker=m, limit=50,
                        sort_dirs='desc'
                    )
                    if the_list:
                        for the_item in the_list:
                            yield the_item
                        m = the_list[-1].id
                    else:
                        return
                except StopIteration:
                    return

        def execution_list_generator():
            yielded = 0
            for sexec in mistral_execution_generator():
                if (sexec.workflow_name == cron_trigger.workflow_name and
                        ct_input == json.loads(sexec.input)):
                    yield ScheduleExecution(self, sexec.to_dict(),
                                            loaded=True)
                    yielded += 1
                if limit and yielded == limit:
                    return

        return list(execution_list_generator())

    def execution_delete(self, execution, mistral_client=None):
        """Remove a given schedule execution.

        :param id: id of execution to remove.
        """

        exec_id = (execution.id if isinstance(execution, ScheduleExecution)
                   else execution)

        if isinstance(execution, ScheduleExecution):
            execution = execution.name

        if not mistral_client:
            mistral_client = self._get_mistral_client()

        mistral_client.executions.delete(exec_id)



#    Copyright 2011 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import json
import optparse
import os
import pickle
import sys
from urllib import parse

from troveclient.compat import client
from troveclient.compat import exceptions


def methods_of(obj):
    """Get all callable methods of an object that don't start with underscore
    returns a list of tuples of the form (method_name, method).
    """
    result = {}
    for i in dir(obj):
        if callable(getattr(obj, i)) and not i.startswith('_'):
            result[i] = getattr(obj, i)
    return result


def check_for_exceptions(resp, body):
    if resp.status in (400, 422, 500):
        raise exceptions.from_response(resp, body)


def print_actions(cmd, actions):
    """Print help for the command with list of options and description."""
    print("Available actions for '%s' cmd:" % cmd)
    for k, v in actions.items():
        print("\t%-20s%s" % (k, v.__doc__))
    sys.exit(2)


def print_commands(commands):
    """Print the list of available commands and description."""

    print("Available commands")
    for k, v in commands.items():
        print("\t%-20s%s" % (k, v.__doc__))
    sys.exit(2)


def limit_url(url, limit=None, marker=None):
    if not limit and not marker:
        return url
    query = []
    if marker:
        query.append("marker=%s" % marker)
    if limit:
        query.append("limit=%s" % limit)
    query = '?' + '&'.join(query)
    return url + query


def quote_user_host(user, host):
    quoted = ''
    if host:
        quoted = parse.quote("%s@%s" % (user, host))
    else:
        quoted = parse.quote("%s" % user)
    return quoted.replace('.', '%2e')


class CliOptions(object):
    """A token object containing the user, apikey and token which
       is pickleable.
    """

    APITOKEN = os.path.expanduser("~/.apitoken")

    DEFAULT_VALUES = {
        'username': None,
        'apikey': None,
        'tenant_id': None,
        'auth_url': None,
        'auth_type': 'keystone',
        'service_type': 'database',
        'service_name': '',
        'region': 'RegionOne',
        'service_url': None,
        'insecure': False,
        'verbose': False,
        'debug': False,
        'token': None,
    }

    def __init__(self, **kwargs):
        for key, value in self.DEFAULT_VALUES.items():
            setattr(self, key, value)

    @classmethod
    def default(cls):
        kwargs = copy.deepcopy(cls.DEFAULT_VALUES)
        return cls(**kwargs)

    @classmethod
    def load_from_file(cls):
        try:
            with open(cls.APITOKEN, 'rb') as token:
                return pickle.load(token)
        except IOError:
            pass  # File probably not found.
        except Exception:
            print("ERROR: Token file found at %s was corrupt." % cls.APITOKEN)
        return cls.default()

    @classmethod
    def save_from_instance_fields(cls, instance):
        apitoken = cls.default()
        for key, default_value in cls.DEFAULT_VALUES.items():
            final_value = getattr(instance, key, default_value)
            setattr(apitoken, key, final_value)
        with open(cls.APITOKEN, 'wb') as token:
            pickle.dump(apitoken, token, protocol=2)

    @classmethod
    def create_optparser(cls, load_file):
        oparser = optparse.OptionParser(
            usage="%prog [options] <cmd> <action> <args>",
            version='1.0', conflict_handler='resolve')
        if load_file:
            file = cls.load_from_file()
        else:
            file = cls.default()

        def add_option(*args, **kwargs):
            if len(args) == 1:
                name = args[0]
            else:
                name = args[1]
            kwargs['default'] = getattr(file, name, cls.DEFAULT_VALUES[name])
            oparser.add_option("--%s" % name, **kwargs)

        add_option("verbose", action="store_true",
                   help="Show equivalent curl statement along "
                        "with actual HTTP communication.")
        add_option("debug", action="store_true",
                   help="Show the stack trace on errors.")
        add_option("auth_url", help="Auth API endpoint URL with port and "
                   "version. Default: http://localhost:5000/v2.0")
        add_option("username", help="Login username.")
        add_option("apikey", help="API key.")
        add_option("tenant_id",
                   help="Tenant Id associated with the account.")
        add_option("auth_type",
                   help="Auth type to support different auth environments, "
                        "Supported value are 'keystone'.")
        add_option("service_type",
                   help="Service type is a name associated for the catalog.")
        add_option("service_name",
                   help="Service name as provided in the service catalog.")
        add_option("service_url",
                   help="Service endpoint to use "
                        "if the catalog doesn't have one.")
        add_option("region", help="Region the service is located in.")
        add_option("insecure", action="store_true",
                   help="Run in insecure mode for https endpoints.")
        add_option("token", help="Token from a prior login.")

        oparser.add_option("--json", action="store_false", dest="xml",
                           help="Changes format to JSON.")
        oparser.add_option("--secure", action="store_false", dest="insecure",
                           help="Run in insecure mode for https endpoints.")
        oparser.add_option("--terse", action="store_false", dest="verbose",
                           help="Toggles verbose mode off.")
        oparser.add_option("--hide-debug", action="store_false", dest="debug",
                           help="Toggles debug mode off.")
        return oparser


class ArgumentRequired(Exception):
    def __init__(self, param):
        self.param = param

    def __str__(self):
        return 'Argument "--%s" required.' % self.param


class ArgumentsRequired(ArgumentRequired):
    def __init__(self, *params):
        self.params = params

    def __str__(self):
        returnstring = 'Specify at least one of these arguments: '
        for param in self.params:
            returnstring = returnstring + '"--%s" ' % param
        return returnstring


class CommandsBase(object):
    params = []

    def __init__(self, parser):
        self._parse_options(parser)

    def _get_client(self):
        """Creates the all important client object."""
        try:
            client_cls = client.TroveHTTPClient
            if self.verbose:
                client.log_to_streamhandler(sys.stdout)
                client.RDC_PP = True
            return client.Dbaas(self.username, self.apikey, self.tenant_id,
                                auth_url=self.auth_url,
                                auth_strategy=self.auth_type,
                                service_type=self.service_type,
                                service_name=self.service_name,
                                region_name=self.region,
                                service_url=self.service_url,
                                insecure=self.insecure,
                                client_cls=client_cls)
        except Exception:
            if self.debug:
                raise
            print(sys.exc_info()[1])

    def _safe_exec(self, func, *args, **kwargs):
        if not self.debug:
            try:
                return func(*args, **kwargs)
            except Exception:
                print(sys.exc_info()[1])
                return None
        else:
            return func(*args, **kwargs)

    @classmethod
    def _prepare_parser(cls, parser):
        for param in cls.params:
            parser.add_option("--%s" % param)

    def _parse_options(self, parser):
        opts, args = parser.parse_args()
        for param in opts.__dict__:
            value = getattr(opts, param)
            setattr(self, param, value)

    def _require(self, *params):
        for param in params:
            if not hasattr(self, param):
                raise ArgumentRequired(param)
            if not getattr(self, param):
                raise ArgumentRequired(param)

    def _require_at_least_one_of(self, *params):
        # One or more of params is required to be present.
        argument_present = False
        for param in params:
            if hasattr(self, param):
                if getattr(self, param):
                    argument_present = True
        if argument_present is False:
            raise ArgumentsRequired(*params)

    def _make_list(self, *params):
        # Convert the listed params to lists.
        for param in params:
            raw = getattr(self, param)
            if isinstance(raw, list):
                return
            raw = [item.strip() for item in raw.split(',')]
            setattr(self, param, raw)

    def _pretty_print(self, func, *args, **kwargs):
        if self.verbose:
            self._safe_exec(func, *args, **kwargs)
            return  # Skip this, since the verbose stuff will show up anyway.

        def wrapped_func():
            result = func(*args, **kwargs)
            if result:
                print(json.dumps(result._info, sort_keys=True, indent=4))
            else:
                print("OK")

        self._safe_exec(wrapped_func)

    def _dumps(self, item):
        return json.dumps(item, sort_keys=True, indent=4)

    def _pretty_list(self, func, *args, **kwargs):
        result = self._safe_exec(func, *args, **kwargs)
        if self.verbose:
            return
        if result and len(result) > 0:
            for item in result:
                print(self._dumps(item._info))
        else:
            print("OK")

    def _pretty_paged(self, func, *args, **kwargs):
        try:
            limit = self.limit
            if limit:
                limit = int(limit, 10)
            result = func(*args, limit=limit, marker=self.marker, **kwargs)
            if self.verbose:
                return  # Verbose already shows the output, so skip this.
            if result and len(result) > 0:
                for item in result:
                    print(self._dumps(item._info))
                if result.links:
                    print("Links:")
                    for link in result.links:
                        print(self._dumps((link)))
            else:
                print("OK")
        except Exception:
            if self.debug:
                raise
            print(sys.exc_info()[1])


class Auth(CommandsBase):
    """Authenticate with your username and api key."""
    params = [
        'apikey',
        'auth_strategy',
        'auth_type',
        'auth_url',
        'options',
        'region',
        'service_name',
        'service_type',
        'service_url',
        'tenant_id',
        'username',
    ]

    def __init__(self, parser):
        super(Auth, self).__init__(parser)
        self.dbaas = None

    def login(self):
        """Login to retrieve an auth token to use for other api calls."""
        self._require('username', 'apikey', 'tenant_id', 'auth_url')
        try:
            self.dbaas = self._get_client()
            self.dbaas.authenticate()
            self.token = self.dbaas.client.auth_token
            self.service_url = self.dbaas.client.service_url
            CliOptions.save_from_instance_fields(self)
            print("Token acquired! Saving to %s..." % CliOptions.APITOKEN)
            print("    service_url = %s" % self.service_url)
            print("    token       = %s" % self.token)
        except Exception:
            if self.debug:
                raise
            print(sys.exc_info()[1])


class AuthedCommandsBase(CommandsBase):
    """Commands that work only with an authenticated client."""

    def __init__(self, parser):
        """Makes sure a token is available somehow and logs in."""
        super(AuthedCommandsBase, self).__init__(parser)
        try:
            self._require('token')
        except ArgumentRequired:
            if self.debug:
                raise
            print('No token argument supplied. Use the "auth login" command '
                  'to log in and get a token.\n')
            sys.exit(1)
        try:
            self._require('service_url')
        except ArgumentRequired:
            if self.debug:
                raise
            print('No service_url given.\n')
            sys.exit(1)
        self.dbaas = self._get_client()
        # Actually set the token to avoid a re-auth.
        self.dbaas.client.auth_token = self.token
        self.dbaas.client.authenticate_with_token(self.token, self.service_url)


class Paginated(object):
    """Pretends to be a list if you iterate over it, but also keeps a
       next property you can use to get the next page of data.
    """

    def __init__(self, items=None, next_marker=None, links=None):
        self.items = items or []
        self.next = next_marker
        self.links = links or []

    def __len__(self):
        return len(self.items)

    def __iter__(self):
        return self.items.__iter__()

    def __getitem__(self, key):
        return self.items[key]

    def __setitem__(self, key, value):
        self.items[key] = value

    def __delitem__(self, key):
        del self.items[key]

    def __reversed__(self):
        return reversed(self.items)

    def __contains__(self, needle):
        return needle in self.items



# Copyright 2014 Tesora Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from troveclient.v1 import databases

"""
Unit tests for databases.py
"""


class DatabasesTest(testtools.TestCase):
    def setUp(self):
        super(DatabasesTest, self).setUp()
        self.orig__init = databases.Databases.__init__
        databases.Databases.__init__ = mock.Mock(return_value=None)
        self.databases = databases.Databases()
        self.databases.api = mock.Mock()
        self.databases.api.client = mock.Mock()

        self.instance_with_id = mock.Mock()
        self.instance_with_id.id = 215

        self.fakedb1 = ['db1']
        self.fakedb2 = ['db1', 'db2']

    def tearDown(self):
        super(DatabasesTest, self).tearDown()
        databases.Databases.__init__ = self.orig__init

    def _get_mock_method(self):
        self._resp = mock.Mock()
        self._body = None
        self._url = None

        def side_effect_func(url, body=None):
            self._body = body
            self._url = url
            return (self._resp, body)

        return mock.Mock(side_effect=side_effect_func)

    def test_create(self):
        self.databases.api.client.post = self._get_mock_method()
        self._resp.status_code = 200

        self.databases.create(23, self.fakedb1)
        self.assertEqual('/instances/23/databases', self._url)
        self.assertEqual({"databases": self.fakedb1}, self._body)

        self.databases.create(23, self.fakedb2)
        self.assertEqual('/instances/23/databases', self._url)
        self.assertEqual({"databases": self.fakedb2}, self._body)

        # test creation with the instance as an object
        self.databases.create(self.instance_with_id, self.fakedb1)
        self.assertEqual({"databases": self.fakedb1}, self._body)

    def test_delete(self):
        self.databases.api.client.delete = self._get_mock_method()
        self._resp.status_code = 200
        self.databases.delete(27, self.fakedb1[0])
        self.assertEqual('/instances/27/databases/%s' % self.fakedb1[0],
                         self._url)
        self.databases.delete(self.instance_with_id, self.fakedb1[0])
        self.assertEqual('/instances/%s/databases/%s' %
                         (self.instance_with_id.id, self.fakedb1[0]),
                         self._url)
        self._resp.status_code = 400
        self.assertRaises(Exception, self.databases.delete, 34, self.fakedb1)

    def test_list(self):
        page_mock = mock.Mock()
        self.databases._paginated = page_mock
        self.databases.list('instance1')
        page_mock.assert_called_with('/instances/instance1/databases',
                                     'databases', None, None)
        limit = 'test-limit'
        marker = 'test-marker'
        self.databases.list('instance1', limit, marker)
        page_mock.assert_called_with('/instances/instance1/databases',
                                     'databases', limit, marker)



#!/usr/bin/env python

#    Copyright 2011 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Trove Command line tool
"""

import os
import sys

from troveclient.compat import common

# If ../trove/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                                os.pardir,
                                                os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'troveclient',
                               '__init__.py')):
    sys.path.insert(0, possible_topdir)


class InstanceCommands(common.AuthedCommandsBase):
    """Commands to perform various instance operations and actions."""

    params = [
        'flavor',
        'id',
        'limit',
        'marker',
        'name',
        'size',
        'backup',
        'availability_zone',
        'configuration_id',
    ]

    def _get_configuration_ref(self):
        configuration_ref = None
        if self.configuration_id is not None:
            if self.configuration_id == "":
                configuration_ref = self.configuration_id
            else:
                configuration_ref = "/".join(
                    [
                        self.dbaas.client.service_url,
                        self.configuration_id,
                    ]
                )
        return configuration_ref

    def create(self):
        """Create a new instance."""
        self._require('name', 'flavor')
        volume = None
        if self.size:
            volume = {"size": self.size}
        restorePoint = None
        if self.backup:
            restorePoint = {"backupRef": self.backup}
        self._pretty_print(self.dbaas.instances.create, self.name,
                           self.flavor, volume, restorePoint=restorePoint,
                           availability_zone=self.availability_zone,
                           configuration=self._get_configuration_ref())

    # TODO(pdmars): is this actually what this should be named?
    def modify(self):
        """Modify an instance."""
        self._require('id')
        self._pretty_print(self.dbaas.instances.modify, self.id,
                           configuration=self._get_configuration_ref())

    def delete(self):
        """Delete the specified instance."""
        self._require('id')
        print(self.dbaas.instances.delete(self.id))

    def get(self):
        """Get details for the specified instance."""
        self._require('id')
        self._pretty_print(self.dbaas.instances.get, self.id)

    def backups(self):
        """Get a list of backups for the specified instance."""
        self._require('id')
        self._pretty_list(self.dbaas.instances.backups, self.id)

    def list(self):
        """List all instances for account."""
        # limit and marker are not required.
        limit = self.limit or None
        if limit:
            limit = int(limit, 10)
        self._pretty_paged(self.dbaas.instances.list)

    def resize_volume(self):
        """Resize an instance volume."""
        self._require('id', 'size')
        self._pretty_print(self.dbaas.instances.resize_volume, self.id,
                           self.size)

    def resize_instance(self):
        """Resize an instance flavor"""
        self._require('id', 'flavor')
        self._pretty_print(self.dbaas.instances.resize_instance, self.id,
                           self.flavor)

    def restart(self):
        """Restart the database."""
        self._require('id')
        self._pretty_print(self.dbaas.instances.restart, self.id)

    def configuration(self):
        """Get configuration for the specified instance."""
        self._require('id')
        self._pretty_print(self.dbaas.instances.configuration, self.id)


class FlavorsCommands(common.AuthedCommandsBase):
    """Command for listing Flavors."""

    params = []

    def list(self):
        """List the available flavors."""
        self._pretty_list(self.dbaas.flavors.list)


class DatabaseCommands(common.AuthedCommandsBase):
    """Database CRUD operations on an instance."""

    params = [
        'name',
        'id',
        'limit',
        'marker',
    ]

    def create(self):
        """Create a database."""
        self._require('id', 'name')
        databases = [{'name': self.name}]
        print(self.dbaas.databases.create(self.id, databases))

    def delete(self):
        """Delete a database."""
        self._require('id', 'name')
        print(self.dbaas.databases.delete(self.id, self.name))

    def list(self):
        """List the databases."""
        self._require('id')
        self._pretty_paged(self.dbaas.databases.list, self.id)


class UserCommands(common.AuthedCommandsBase):
    """User CRUD operations on an instance."""
    params = [
        'id',
        'database',
        'databases',
        'hostname',
        'name',
        'password',
        'new_name',
        'new_host',
        'new_password',
    ]

    def create(self):
        """Create a user in instance, with access to one or more databases."""
        self._require('id', 'name', 'password', 'databases')
        self._make_list('databases')
        databases = [{'name': dbname} for dbname in self.databases]
        users = [{'name': self.name, 'password': self.password,
                  'databases': databases}]
        if self.hostname:
            users[0]['host'] = self.hostname
        self.dbaas.users.create(self.id, users)

    def delete(self):
        """Delete the specified user"""
        self._require('id', 'name')
        self.dbaas.users.delete(self.id, self.name, self.hostname)

    def get(self):
        """Get a single user."""
        self._require('id', 'name')
        self._pretty_print(self.dbaas.users.get, self.id,
                           self.name, self.hostname)

    def update_attributes(self):
        """Update attributes of a single user."""
        self._require('id', 'name')
        self._require_at_least_one_of('new_name', 'new_host', 'new_password')
        user_new = {}
        if self.new_name:
            user_new['name'] = self.new_name
        if self.new_host:
            user_new['host'] = self.new_host
        if self.new_password:
            user_new['password'] = self.new_password
        self.dbaas.users.update_attributes(self.id, self.name, user_new,
                                           self.hostname)

    def list(self):
        """List all the users for an instance."""
        self._require('id')
        self._pretty_paged(self.dbaas.users.list, self.id)

    def access(self):
        """Show all databases the user has access to."""
        self._require('id', 'name')
        self._pretty_list(self.dbaas.users.list_access, self.id,
                          self.name, self.hostname)

    def grant(self):
        """Allow an existing user permissions to access one or more
        databases.
        """
        self._require('id', 'name', 'databases')
        self._make_list('databases')
        self.dbaas.users.grant(self.id, self.name, self.databases,
                               self.hostname)

    def revoke(self):
        """Revoke from an existing user access permissions to a database."""
        self._require('id', 'name', 'database')
        self.dbaas.users.revoke(self.id, self.name, self.database,
                                self.hostname)

    def change_password(self):
        """Change the password of a single user."""
        self._require('id', 'name', 'password')
        users = [{'name': self.name,
                  'host': self.hostname,
                  'password': self.password}]
        self.dbaas.users.change_passwords(self.id, users)


class RootCommands(common.AuthedCommandsBase):
    """Root user related operations on an instance."""

    params = [
        'id',
    ]

    def create(self):
        """Enable the instance's root user."""
        self._require('id')
        try:
            user, password = self.dbaas.root.create(self.id)
            print("User:\t\t%s\nPassword:\t%s" % (user, password))
        except Exception:
            print(sys.exc_info()[1])

    def delete(self):
        """Disable the instance's root user."""
        self._require('id')
        print(self.dbaas.root.delete(self.id))

    def enabled(self):
        """Check the instance for root access."""
        self._require('id')
        self._pretty_print(self.dbaas.root.is_root_enabled, self.id)


class VersionCommands(common.AuthedCommandsBase):
    """List available versions."""

    params = [
        'url',
    ]

    def list(self):
        """List all the supported versions."""
        self._require('url')
        self._pretty_list(self.dbaas.versions.index, self.url)


class LimitsCommands(common.AuthedCommandsBase):
    """Show the rate limits and absolute limits."""

    def list(self):
        """List the rate limits and absolute limits."""
        self._pretty_list(self.dbaas.limits.list)


class BackupsCommands(common.AuthedCommandsBase):
    """Command to manage and show backups."""
    params = ['name', 'instance', 'description']

    def get(self):
        """Get details for the specified backup."""
        self._require('id')
        self._pretty_print(self.dbaas.backups.get, self.id)

    def list(self):
        """List backups."""
        self._pretty_list(self.dbaas.backups.list)

    def create(self):
        """Create a new backup."""
        self._require('name', 'instance')
        self._pretty_print(self.dbaas.backups.create, self.name,
                           self.instance, self.description)

    def delete(self):
        """Delete a backup."""
        self._require('id')
        self._pretty_print(self.dbaas.backups.delete, self.id)


class DatastoreConfigurationParameters(common.AuthedCommandsBase):
    """Command to show configuration parameters for a datastore."""
    params = ['datastore', 'parameter']

    def parameters(self):
        """List parameters that can be set."""
        self._require('datastore')
        self._pretty_print(self.dbaas.configuration_parameters.parameters,
                           self.datastore)

    def get_parameter(self):
        """List parameters that can be set."""
        self._require('datastore', 'parameter')
        self._pretty_print(self.dbaas.configuration_parameters.get_parameter,
                           self.datastore, self.parameter)


class ConfigurationsCommands(common.AuthedCommandsBase):
    """Command to manage and show configurations."""
    params = ['name', 'instances', 'values', 'description', 'parameter']

    def get(self):
        """Get details for the specified configuration."""
        self._require('id')
        self._pretty_print(self.dbaas.configurations.get, self.id)

    def list_instances(self):
        """Get details for the specified configuration."""
        self._require('id')
        self._pretty_list(self.dbaas.configurations.instances, self.id)

    def list(self):
        """List configurations."""
        self._pretty_list(self.dbaas.configurations.list)

    def create(self):
        """Create a new configuration."""
        self._require('name', 'values')
        self._pretty_print(self.dbaas.configurations.create, self.name,
                           self.values, self.description)

    def update(self):
        """Update an existing configuration."""
        self._require('id', 'values')
        self._pretty_print(self.dbaas.configurations.update, self.id,
                           self.values, self.name, self.description)

    def edit(self):
        """Edit an existing configuration values."""
        self._require('id', 'values')
        self._pretty_print(self.dbaas.configurations.edit, self.id,
                           self.values)

    def delete(self):
        """Delete a configuration."""
        self._require('id')
        self._pretty_print(self.dbaas.configurations.delete, self.id)


class SecurityGroupCommands(common.AuthedCommandsBase):
    """Commands to list and show Security Groups For an Instance and
    create and delete security group rules for them.
    """
    params = [
        'id',
        'secgroup_id',
        'protocol',
        'from_port',
        'to_port',
        'cidr'
    ]

    def get(self):
        """Get a security group associated with an instance."""
        self._require('id')
        self._pretty_print(self.dbaas.security_groups.get, self.id)

    def list(self):
        """List all the Security Groups and the rules."""
        self._pretty_paged(self.dbaas.security_groups.list)

    def add_rule(self):
        """Add a security group rule."""
        self._require('secgroup_id', 'protocol',
                      'from_port', 'to_port', 'cidr')
        self.dbaas.security_group_rules.create(self.secgroup_id, self.protocol,
                                               self.from_port, self.to_port,
                                               self.cidr)

    def delete_rule(self):
        """Delete a security group rule."""
        self._require('id')
        self.dbaas.security_group_rules.delete(self.id)


class MetadataCommands(common.AuthedCommandsBase):
    """Commands to create/update/replace/delete/show metadata for an instance
    """
    params = [
        'instance_id',
        'metadata'
    ]

    def show(self):
        """Show instance metadata."""
        self._require('instance_id')
        self._pretty_print(self.dbaas.metadata.show(self.instance_id))


COMMANDS = {
    'auth': common.Auth,
    'instance': InstanceCommands,
    'flavor': FlavorsCommands,
    'database': DatabaseCommands,
    'limit': LimitsCommands,
    'backup': BackupsCommands,
    'configuration': ConfigurationsCommands,
    'user': UserCommands,
    'root': RootCommands,
    'version': VersionCommands,
    'secgroup': SecurityGroupCommands,
    'metadata': MetadataCommands,
}


def main():
    # Parse arguments
    load_file = True
    for index, arg in enumerate(sys.argv):
        if (arg == "auth" and len(sys.argv) > (index + 1) and
                sys.argv[index + 1] == "login"):
            load_file = False

    oparser = common.CliOptions.create_optparser(load_file)
    for k, v in COMMANDS.items():
        v._prepare_parser(oparser)
    (options, args) = oparser.parse_args()

    if not args:
        common.print_commands(COMMANDS)

    if options.verbose:
        os.environ['RDC_PP'] = "True"
        os.environ['REDDWARFCLIENT_DEBUG'] = "True"

    # Pop the command and check if it's in the known commands
    cmd = args.pop(0)
    if cmd in COMMANDS:
        fn = COMMANDS.get(cmd)
        command_object = None
        try:
            command_object = fn(oparser)
        except Exception as ex:
            if options.debug:
                raise
            print(ex)

        # Get a list of supported actions for the command
        actions = common.methods_of(command_object)

        if len(args) < 1:
            common.print_actions(cmd, actions)

        # Check for a valid action and perform that action
        action = args.pop(0)
        if action in actions:
            if not options.debug:
                try:
                    getattr(command_object, action)()
                except Exception as ex:
                    if options.debug:
                        raise
                    print(ex)
            else:
                getattr(command_object, action)()
        else:
            common.print_actions(cmd, actions)
    else:
        common.print_commands(COMMANDS)


if __name__ == '__main__':
    main()



#    Copyright 2011 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class UnsupportedVersion(Exception):
    """Indicates that the user is trying to use an unsupported
    version of the API.
    """
    pass


class CommandError(Exception):
    pass


class AuthorizationFailure(Exception):
    pass


class NoUniqueMatch(Exception):
    pass


class NoTokenLookupException(Exception):
    """This form of authentication does not support looking up
       endpoints from an existing token.
    """
    pass


class EndpointNotFound(Exception):
    """Could not find Service or Region in Service Catalog."""
    pass


class AuthUrlNotGiven(EndpointNotFound):
    """The auth url was not given."""
    pass


class ServiceUrlNotGiven(EndpointNotFound):
    """The service url was not given."""
    pass


class ResponseFormatError(Exception):
    """Could not parse the response format."""
    pass


class AmbiguousEndpoints(Exception):
    """Found more than one matching endpoint in Service Catalog."""
    def __init__(self, endpoints=None):
        self.endpoints = endpoints

    def __str__(self):
        return "AmbiguousEndpoints: %s" % repr(self.endpoints)


class ClientException(Exception):
    """The base exception class for all exceptions this library raises."""
    def __init__(self, code, message=None, details=None, request_id=None):
        self.code = code
        self.message = message or self.__class__.message
        self.details = details
        self.request_id = request_id

    def __str__(self):
        formatted_string = "%s (HTTP %s)" % (self.message, self.code)
        if self.request_id:
            formatted_string += " (Request-ID: %s)" % self.request_id

        return formatted_string


class BadRequest(ClientException):
    """HTTP 400 - Bad request: you sent some malformed data."""
    http_status = 400
    message = "Bad request"


class Unauthorized(ClientException):
    """HTTP 401 - Unauthorized: bad credentials."""
    http_status = 401
    message = "Unauthorized"


class Forbidden(ClientException):
    """HTTP 403 - Forbidden: your don't have access to this resource."""
    http_status = 403
    message = "Forbidden"


class NotFound(ClientException):
    """HTTP 404 - Not found."""
    http_status = 404
    message = "Not found"


class Conflict(ClientException):
    """HTTP 409 - Conflict."""
    http_status = 409
    message = "Conflict"


class OverLimit(ClientException):
    """HTTP 413
    - Over limit: you're over the API limits for this time period.
    """
    http_status = 413
    message = "Over limit"


# NotImplemented is a python keyword.
class HTTPNotImplemented(ClientException):
    """HTTP 501
    - Not Implemented: the server does not support this operation.
    """
    http_status = 501
    message = "Not Implemented"


class UnprocessableEntity(ClientException):
    """HTTP 422 - Unprocessable Entity: The request cannot be processed."""
    http_status = 422
    message = "Unprocessable Entity"


# In Python 2.4 Exception is old-style and thus doesn't have a __subclasses__()
# so we can do this:
#     _code_map = dict((c.http_status, c)
#                      for c in ClientException.__subclasses__())
#
# Instead, we have to hardcode it:
_code_map = dict((c.http_status, c) for c in [BadRequest, Unauthorized,
                                              Forbidden, NotFound, Conflict,
                                              OverLimit, HTTPNotImplemented,
                                              UnprocessableEntity])


def from_response(response, body):
    """Return an instance of an ClientException based on a request's response.

    Usage::

        resp, body = http.request(...)
        if resp.status != 200:
            raise exception_from_response(resp, body)
    """
    cls = _code_map.get(response.status, ClientException)
    if body:
        message = "n/a"
        details = "n/a"
        if hasattr(body, 'keys'):
            error = body[list(body.keys())[0]]
            message = error.get('message', None)
            details = error.get('details', None)
        return cls(code=response.status, message=message, details=details)
    else:
        request_id = response.get('x-compute-request-id')
        return cls(code=response.status, request_id=request_id)



# Copyright 2011 OpenStack Foundation
# Copyright 2011, Piston Cloud Computing, Inc.
# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from troveclient.apiclient import exceptions


class ServiceCatalog(object):
    """Helper methods for dealing with a Keystone Service Catalog."""

    def __init__(self, resource_dict):
        self.catalog = resource_dict

    def get_token(self):
        return self.catalog['access']['token']['id']

    def url_for(self, attr=None, filter_value=None,
                service_type=None, endpoint_type='publicURL',
                service_name=None, database_service_name=None):
        """Fetch the public URL from the Compute service for
        a particular endpoint attribute. If none given, return
        the first. See tests for sample service catalog.
        """
        matching_endpoints = []
        if 'endpoints' in self.catalog:
            # We have a bastardized service catalog. Treat it special. :/
            for endpoint in self.catalog['endpoints']:
                if not filter_value or endpoint[attr] == filter_value:
                    matching_endpoints.append(endpoint)
            if not matching_endpoints:
                raise exceptions.EndpointNotFound()

        # We don't always get a service catalog back ...
        if 'serviceCatalog' not in self.catalog['access']:
            return None

        # Full catalog ...
        catalog = self.catalog['access']['serviceCatalog']

        for service in catalog:

            # NOTE(thingee): For backwards compatibility, if they have v2
            # enabled and the service_type is set to 'database', go ahead and
            # accept that.
            skip_service_type_check = False
            if service_type == 'databasev2' and service['type'] == 'database':
                version = service['endpoints'][0]['publicURL'].split('/')[3]
                if version == 'v2':
                    skip_service_type_check = True

            if (not skip_service_type_check and
                    service.get("type") != service_type):
                continue

            if (database_service_name and service_type in ('database',
                                                           'databasev2') and
                    service.get('name') != database_service_name):
                continue

            endpoints = service['endpoints']
            for endpoint in endpoints:
                if not filter_value or endpoint.get(attr) == filter_value:
                    endpoint["serviceName"] = service.get("name")
                    matching_endpoints.append(endpoint)

        if not matching_endpoints:
            raise exceptions.EndpointNotFound()
        elif len(matching_endpoints) > 1:
            raise exceptions.AmbiguousEndpoints(
                endpoints=matching_endpoints)
        else:
            return matching_endpoints[0][endpoint_type]



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib
import os
from unittest import mock

import testtools

from troveclient.apiclient import exceptions
from troveclient import base
from troveclient import common
from troveclient import utils

"""
Unit tests for base.py
"""

UUID = '8e8ec658-c7b0-4243-bdf8-6f7f2952c0d0'


def obj_class(self, res, loaded=True):
    return res


class BaseTest(testtools.TestCase):
    def test_getid(self):
        obj = "test"
        r = base.getid(obj)
        self.assertEqual(obj, r)

        test_id = "test_id"
        obj = mock.Mock()
        obj.id = test_id
        r = base.getid(obj)
        self.assertEqual(test_id, r)


class ManagerTest(testtools.TestCase):
    def setUp(self):
        super(ManagerTest, self).setUp()
        self.orig__init = base.Manager.__init__
        base.Manager.__init__ = mock.Mock(return_value=None)
        self.orig_os_makedirs = os.makedirs

    def tearDown(self):
        super(ManagerTest, self).tearDown()
        base.Manager.__init__ = self.orig__init
        os.makedirs = self.orig_os_makedirs

    def test___init__(self):
        api = mock.Mock()
        base.Manager.__init__ = self.orig__init
        manager = base.Manager(api)
        self.assertEqual(api, manager.api)

    def test_completion_cache(self):
        manager = base.Manager()

        # handling exceptions
        mode = "w"
        cache_type = "unittest"
        obj_class = mock.Mock
        with manager.completion_cache(cache_type, obj_class, mode):
            pass

        os.makedirs = mock.Mock(side_effect=OSError)
        with manager.completion_cache(cache_type, obj_class, mode):
            pass

    def test_write_to_completion_cache(self):
        manager = base.Manager()

        # no cache object, nothing should happen
        manager.write_to_completion_cache("non-exist", "val")
        manager._mock_cache = mock.Mock()
        manager._mock_cache.write = mock.Mock(return_value=None)
        manager.write_to_completion_cache("mock", "val")
        self.assertEqual(1, manager._mock_cache.write.call_count)

    def _get_mock(self):
        manager = base.Manager()
        manager.api = mock.Mock()
        manager.api.client = mock.Mock()

        def side_effect_func(self, body, loaded=True):
            return body

        manager.resource_class = mock.Mock(side_effect=side_effect_func)
        return manager

    def test__get_with_response_key_none(self):
        manager = self._get_mock()
        url_ = "test-url"
        body_ = "test-body"
        resp_ = "test-resp"
        manager.api.client.get = mock.Mock(return_value=(resp_, body_))
        r = manager._get(url=url_, response_key=None)
        self.assertEqual(body_, r)

    def test__get_with_response_key(self):
        manager = self._get_mock()
        response_key = "response_key"
        body_ = {response_key: "test-resp-key-body"}
        url_ = "test_url_get"
        manager.api.client.get = mock.Mock(return_value=(url_, body_))
        r = manager._get(url=url_, response_key=response_key)
        self.assertEqual(body_[response_key], r)

    def test__create(self):
        manager = base.Manager()
        manager.api = mock.Mock()
        manager.api.client = mock.Mock()

        response_key = "response_key"
        data_ = "test-data"
        body_ = {response_key: data_}
        url_ = "test_url_post"
        manager.api.client.post = mock.Mock(return_value=(url_, body_))

        return_raw = True
        r = manager._create(url_, body_, response_key, return_raw)
        self.assertEqual(data_, r)

        return_raw = False

        @contextlib.contextmanager
        def completion_cache_mock(*arg, **kwargs):
            yield

        mockl = mock.Mock()
        mockl.side_effect = completion_cache_mock
        manager.completion_cache = mockl

        manager.resource_class = mock.Mock(return_value="test-class")
        r = manager._create(url_, body_, response_key, return_raw)
        self.assertEqual("test-class", r)

    def get_mock_mng_api_client(self):
        manager = base.Manager()
        manager.api = mock.Mock()
        manager.api.client = mock.Mock()
        return manager

    def test__delete(self):
        resp_ = "test-resp"
        body_ = "test-body"

        manager = self.get_mock_mng_api_client()
        manager.api.client.delete = mock.Mock(return_value=(resp_, body_))
        # _delete just calls api.client.delete, and does nothing
        # the correctness should be tested in api class
        manager._delete("test-url")
        pass

    def test__update(self):
        resp_ = "test-resp"
        body_ = "test-body"

        manager = self.get_mock_mng_api_client()
        manager.api.client.put = mock.Mock(return_value=(resp_, body_))
        body = manager._update("test-url", body_)
        self.assertEqual(body_, body)


class ManagerListTest(ManagerTest):
    def setUp(self):
        super(ManagerListTest, self).setUp()

        @contextlib.contextmanager
        def completion_cache_mock(*arg, **kwargs):
            yield

        self.manager = base.Manager()
        self.manager.api = mock.Mock()
        self.manager.api.client = mock.Mock()

        self.response_key = "response_key"
        self.data_p = ["p1", "p2"]
        self.body_p = {self.response_key: self.data_p}
        self.url_p = "test_url_post"
        self.manager.api.client.post = mock.Mock(
            return_value=(self.url_p, self.body_p)
        )
        self.data_g = ["g1", "g2", "g3"]
        self.body_g = {self.response_key: self.data_g}
        self.url_g = "test_url_get"
        self.manager.api.client.get = mock.Mock(
            return_value=(self.url_g, self.body_g)
        )

        mockl = mock.Mock()
        mockl.side_effect = completion_cache_mock
        self.manager.completion_cache = mockl

    def tearDown(self):
        super(ManagerListTest, self).tearDown()

    def test_list_with_body_none(self):
        body = None
        li = self.manager._list("url", self.response_key, obj_class, body)
        self.assertEqual(len(self.data_g), len(li))
        for i in range(0, len(li)):
            self.assertEqual(self.data_g[i], li[i])

    def test_list_body_not_none(self):
        body = "something"
        li = self.manager._list("url", self.response_key, obj_class, body)
        self.assertEqual(len(self.data_p), len(li))
        for i in range(0, len(li)):
            self.assertEqual(self.data_p[i], li[i])

    def test_list_key_mapping(self):
        data_ = {"values": ["p1", "p2"]}
        body_ = {self.response_key: data_}
        url_ = "test_url_post"
        self.manager.api.client.post = mock.Mock(return_value=(url_, body_))
        li = self.manager._list("url", self.response_key,
                                obj_class, "something")
        data = data_["values"]
        self.assertEqual(len(data), len(li))
        for i in range(0, len(li)):
            self.assertEqual(data[i], li[i])

    def test_list_without_key_mapping(self):
        data_ = {"v1": "1", "v2": "2"}
        body_ = {self.response_key: data_}
        url_ = "test_url_post"
        self.manager.api.client.post = mock.Mock(return_value=(url_, body_))
        li = self.manager._list("url", self.response_key,
                                obj_class, "something")
        self.assertEqual(len(data_), len(li))


class MangerPaginationTests(ManagerTest):

    def setUp(self):
        super(MangerPaginationTests, self).setUp()
        self.manager = base.Manager()
        self.manager.api = mock.Mock()
        self.manager.api.client = mock.Mock()
        self.manager.resource_class = base.Resource

        self.response_key = "response_key"
        self.data = [{"foo": "p1"}, {"foo": "p2"}]
        self.next_data = [{"foo": "p3"}, {"foo": "p4"}]
        self.marker = 'test-marker'
        self.limit = '20'
        self.url = "http://test_url"
        self.next_url = '%s?marker=%s&limit=%s' % (self.url, self.marker,
                                                   self.limit)
        self.links = [{'href': self.next_url, 'rel': 'next'}]
        self.body = {
            self.response_key: self.data,
            'links': self.links
        }
        self.next_body = {self.response_key: self.next_data}

        def side_effect(url):
            if url == self.url:
                return None, self.body
            # In python 3 the order in the dictionary is not constant
            # between runs. So we cant rely on the URL params to be
            # in the same order
            if ('marker=%s' % self.marker in url and
                    'limit=%s' % self.limit in url):
                self.next_url = url
                return None, self.next_body

        self.manager.api.client.get = mock.Mock(side_effect=side_effect)

    def tearDown(self):
        super(MangerPaginationTests, self).tearDown()

    def test_pagination(self):
        resp = self.manager._paginated(self.url, self.response_key)
        self.manager.api.client.get.assert_called_with(self.url)
        self.assertEqual('p1', resp[0].foo)
        self.assertEqual('p2', resp[1].foo)
        self.assertEqual(self.marker, resp.next)
        self.assertEqual(self.links, resp.links)
        self.assertIsInstance(resp, common.Paginated)

    def test_pagination_next(self):
        resp = self.manager._paginated(self.url, self.response_key,
                                       limit=self.limit, marker=self.marker)
        self.manager.api.client.get.assert_called_with(self.next_url)
        self.assertEqual('p3', resp[0].foo)
        self.assertEqual('p4', resp[1].foo)
        self.assertIsNone(resp.next)
        self.assertEqual([], resp.links)
        self.assertIsInstance(resp, common.Paginated)

    def test_pagination_error(self):
        self.manager.api.client.get = mock.Mock(return_value=(None, None))
        self.assertRaises(Exception, self.manager._paginated,
                          self.url, self.response_key)


class FakeResource(object):
    def __init__(self, _id, properties):
        self.id = _id
        try:
            self.name = properties['name']
        except KeyError:
            pass
        try:
            self.display_name = properties['display_name']
        except KeyError:
            pass


class FakeManager(base.ManagerWithFind):
    resource_class = FakeResource

    resources = [
        FakeResource('1234', {'name': 'entity_one'}),
        FakeResource(UUID, {'name': 'entity_two'}),
        FakeResource('4242', {'display_name': 'entity_three'}),
        FakeResource('5678', {'name': '9876'})
    ]

    def get(self, resource_id):
        for resource in self.resources:
            if resource.id == str(resource_id):
                return resource
        raise exceptions.NotFound(resource_id)

    def list(self):
        return self.resources


class FindResourceTestCase(testtools.TestCase):
    def setUp(self):
        super(FindResourceTestCase, self).setUp()
        self.manager = FakeManager(None)

    def test_find_none(self):
        self.assertRaises(exceptions.CommandError,
                          utils.find_resource,
                          self.manager,
                          'asdf')

    def test_find_by_integer_id(self):
        output = utils.find_resource(self.manager, 1234)
        self.assertEqual(self.manager.get('1234'), output)

    def test_find_by_str_id(self):
        output = utils.find_resource(self.manager, '1234')
        self.assertEqual(self.manager.get('1234'), output)

    def test_find_by_uuid(self):
        output = utils.find_resource(self.manager, UUID)
        self.assertEqual(self.manager.get(UUID), output)

    def test_find_by_str_name(self):
        output = utils.find_resource(self.manager, 'entity_one')
        self.assertEqual(self.manager.get('1234'), output)

    def test_find_by_str_displayname(self):
        output = utils.find_resource(self.manager, 'entity_three')
        self.assertEqual(self.manager.get('4242'), output)

    def test_find_by_int_name(self):
        output = utils.find_resource(self.manager, 9876)
        self.assertEqual(self.manager.get('5678'), output)


class ResourceTest(testtools.TestCase):
    def setUp(self):
        super(ResourceTest, self).setUp()
        self.orig___init__ = base.Resource.__init__

    def tearDown(self):
        super(ResourceTest, self).tearDown()
        base.Resource.__init__ = self.orig___init__

    def test___init__(self):
        manager = mock.Mock()
        manager.write_to_completion_cache = mock.Mock(return_value=None)

        info_ = {}
        robj = base.Resource(manager, info_)
        self.assertEqual(0, manager.write_to_completion_cache.call_count)

        info_ = {"id": "id-with-less-than-36-char"}
        robj = base.Resource(manager, info_)
        self.assertEqual(info_["id"], robj.id)
        self.assertEqual(0, manager.write_to_completion_cache.call_count)

        id_ = "id-with-36-char-"
        for i in range(36 - len(id_)):
            id_ = id_ + "-"
        info_ = {"id": id_}
        robj = base.Resource(manager, info_)
        self.assertEqual(info_["id"], robj.id)
        self.assertEqual(1, manager.write_to_completion_cache.call_count)

        info_["name"] = "test-human-id"
        # Resource.HUMAN_ID is False
        robj = base.Resource(manager, info_)
        self.assertEqual(info_["id"], robj.id)
        self.assertIsNone(robj.human_id)
        self.assertEqual(2, manager.write_to_completion_cache.call_count)

        # base.Resource.HUMAN_ID = True
        info_["HUMAN_ID"] = True
        robj = base.Resource(manager, info_)
        self.assertEqual(info_["id"], robj.id)
        self.assertEqual(info_["name"], robj.human_id)
        self.assertEqual(4, manager.write_to_completion_cache.call_count)

    def test_human_id(self):
        manager = mock.Mock()
        manager.write_to_completion_cache = mock.Mock(return_value=None)

        info_ = {"name": "test-human-id"}
        robj = base.Resource(manager, info_)
        self.assertIsNone(robj.human_id)

        info_["HUMAN_ID"] = True
        robj = base.Resource(manager, info_)
        self.assertEqual(info_["name"], robj.human_id)
        robj.name = "new-human-id"
        self.assertEqual("new-human-id", robj.human_id)

    def get_mock_resource_obj(self):
        base.Resource.__init__ = mock.Mock(return_value=None)
        robj = base.Resource()
        robj._loaded = False
        return robj

    def test__add_details(self):
        robj = self.get_mock_resource_obj()
        info_ = {"name": "test-human-id", "test_attr": 5}
        robj._add_details(info_)
        self.assertEqual(info_["name"], robj.name)
        self.assertEqual(info_["test_attr"], robj.test_attr)

    def test___getattr__(self):
        robj = self.get_mock_resource_obj()
        info_ = {"name": "test-human-id", "test_attr": 5}
        robj._add_details(info_)
        self.assertEqual(info_["test_attr"], robj.__getattr__("test_attr"))

        # TODO(dmakogon): looks like causing infinite recursive calls
        # robj.__getattr__("test_non_exist_attr")

    def test___repr__(self):
        robj = self.get_mock_resource_obj()
        info_ = {"name": "test-human-id", "test_attr": 5}
        robj._add_details(info_)

        expected = "<Resource name=test-human-id, test_attr=5>"
        self.assertEqual(expected, robj.__repr__())

    def test_get(self):
        robj = self.get_mock_resource_obj()
        manager = mock.Mock()
        manager.get = None

        robj.manager = object()
        robj._get()

        manager = mock.Mock()
        robj.manager = mock.Mock()

        robj.id = "id"
        new = mock.Mock()
        new._info = {"name": "test-human-id", "test_attr": 5}
        robj.manager.get = mock.Mock(return_value=new)
        robj._get()
        self.assertEqual("test-human-id", robj.name)
        self.assertEqual(5, robj.test_attr)

    def test___eq__(self):
        robj = self.get_mock_resource_obj()
        other = base.Resource()

        info_ = {"name": "test-human-id", "test_attr": 5}
        robj._info = info_
        other._info = {}
        self.assertFalse(robj.__eq__(other))

        robj.id = "rid"
        other.id = "oid"
        self.assertFalse(robj.__eq__(other))

        other.id = "rid"
        self.assertTrue(robj.__eq__(other))

        # not instance of the same class
        other = mock.Mock()
        self.assertEqual(robj.__eq__(other), NotImplemented)

    def test_is_loaded(self):
        robj = self.get_mock_resource_obj()
        robj._loaded = True
        self.assertTrue(robj.is_loaded)

        robj._loaded = False
        self.assertFalse(robj.is_loaded)



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import testtools
from unittest import mock

from troveclient import base
from troveclient.v1 import datastores


"""
Unit tests for datastores.py
"""


class DatastoreTest(testtools.TestCase):

    def setUp(self):
        super(DatastoreTest, self).setUp()
        self.orig__init = datastores.Datastore.__init__
        datastores.Datastore.__init__ = mock.Mock(return_value=None)
        self.datastore = datastores.Datastore()
        self.datastore.manager = mock.Mock()

    def tearDown(self):
        super(DatastoreTest, self).tearDown()
        datastores.Datastore.__init__ = self.orig__init

    def test___repr__(self):
        self.datastore.name = "datastore-1"
        self.assertEqual('<Datastore: datastore-1>',
                         self.datastore.__repr__())


class DatastoresTest(testtools.TestCase):

    def setUp(self):
        super(DatastoresTest, self).setUp()
        self.orig__init = datastores.Datastores.__init__
        datastores.Datastores.__init__ = mock.Mock(return_value=None)
        self.datastores = datastores.Datastores()
        self.datastores.api = mock.Mock()
        self.datastores.api.client = mock.Mock()
        self.datastores.resource_class = mock.Mock(return_value="ds-1")

        self.orig_base_getid = base.getid
        base.getid = mock.Mock(return_value="datastore1")

    def tearDown(self):
        super(DatastoresTest, self).tearDown()
        datastores.Datastores.__init__ = self.orig__init
        base.getid = self.orig_base_getid

    def test_list(self):
        page_mock = mock.Mock()
        self.datastores._paginated = page_mock
        limit = "test-limit"
        marker = "test-marker"
        self.datastores.list(limit, marker)
        page_mock.assert_called_with("/datastores", "datastores",
                                     limit, marker)
        self.datastores.list()
        page_mock.assert_called_with("/datastores", "datastores", None, None)

    def test_get(self):
        def side_effect_func(path, inst):
            return path, inst

        self.datastores._get = mock.Mock(side_effect=side_effect_func)
        self.assertEqual(('/datastores/datastore1',
                          'datastore'),
                         self.datastores.get(1))


class DatastoreVersionsTest(testtools.TestCase):

    def setUp(self):
        super(DatastoreVersionsTest, self).setUp()
        self.orig__init = datastores.DatastoreVersions.__init__
        datastores.DatastoreVersions.__init__ = mock.Mock(return_value=None)
        self.datastore_versions = datastores.DatastoreVersions()
        self.datastore_versions.api = mock.Mock()
        self.datastore_versions.api.client = mock.Mock()
        self.datastore_versions.resource_class = mock.Mock(
            return_value="ds_version-1")

        self.orig_base_getid = base.getid
        base.getid = mock.Mock(return_value="datastore_version1")

    def tearDown(self):
        super(DatastoreVersionsTest, self).tearDown()
        datastores.DatastoreVersions.__init__ = self.orig__init
        base.getid = self.orig_base_getid

    def test_list(self):
        page_mock = mock.Mock()
        self.datastore_versions._paginated = page_mock
        limit = "test-limit"
        marker = "test-marker"
        self.datastore_versions.list("datastore1", limit, marker)
        page_mock.assert_called_with("/datastores/datastore1/versions",
                                     "versions", limit, marker)

    def test_get(self):
        def side_effect_func(path, inst):
            return path, inst

        self.datastore_versions._get = mock.Mock(side_effect=side_effect_func)
        self.assertEqual(('/datastores/datastore1/versions/'
                          'datastore_version1',
                          'version'),
                         self.datastore_versions.get("datastore1",
                                                     "datastore_version1"))

    def test_get_by_uuid(self):
        def side_effect_func(path, inst):
            return path, inst

        self.datastore_versions._get = mock.Mock(side_effect=side_effect_func)
        self.assertEqual(('/datastores/versions/datastore_version1',
                          'version'),
                         (self.datastore_versions.
                          get_by_uuid("datastore_version1")))


class DatastoreVersionMembersTest(testtools.TestCase):

    def setUp(self):
        super(DatastoreVersionMembersTest, self).setUp()
        self.orig__init = datastores.DatastoreVersionMembers.__init__
        datastores.DatastoreVersionMembers.__init__ = mock.Mock(
            return_value=None)
        self.datastore_version_members = datastores.DatastoreVersionMembers()
        self.datastore_version_members.api = mock.Mock()
        self.datastore_version_members.api.client = mock.Mock()
        self.datastore_version_members.resource_class = mock.Mock(
            return_value="ds_version_member-1")

        self.orig_base_getid = base.getid
        base.getid = mock.Mock(return_value="datastore_version_member1")

    def tearDown(self):
        super(DatastoreVersionMembersTest, self).tearDown()
        datastores.DatastoreVersionMembers.__init__ = self.orig__init
        base.getid = self.orig_base_getid

    def test_add(self):
        def side_effect_func(path, body, inst):
            return path, body, inst

        self.datastore_version_members._create = mock.Mock(
            side_effect=side_effect_func)
        p, b, i = self.datastore_version_members.add("data_store1",
                                                     "datastore_version1",
                                                     "tenant1")
        self.assertEqual(
            "/mgmt/datastores/data_store1/versions/datastore_version1/members",
            p)
        self.assertEqual("datastore_version_member", i)
        self.assertEqual("tenant1", b["member"])

    def test_delete(self):
        def side_effect_func(path):
            return path

        self.datastore_version_members._delete = mock.Mock(
            side_effect=side_effect_func)
        p = self.datastore_version_members.delete("data_store1",
                                                  "datastore_version1",
                                                  "tenant1")
        self.assertEqual(
            "/mgmt/datastores/data_store1/versions/datastore_version1/members/"
            "tenant1",
            p)

    def test_list(self):
        page_mock = mock.Mock()
        self.datastore_version_members._list = page_mock
        limit = "test-limit"
        marker = "test-marker"
        self.datastore_version_members.list("datastore1", "datastore_version1",
                                            limit, marker)
        page_mock.assert_called_with("/mgmt/datastores/datastore1/versions/"
                                     "datastore_version1/members",
                                     "datastore_version_members",
                                     limit, marker)

    def test_get(self):
        def side_effect_func(path, inst):
            return path, inst

        self.datastore_version_members._get = mock.Mock(
            side_effect=side_effect_func)
        self.assertEqual(('/mgmt/datastores/datastore1/versions/'
                          'datastore_version1/members/tenant1',
                          'datastore_version_member'),
                         self.datastore_version_members.get(
                             "datastore1",
                             "datastore_version1",
                             "tenant1"))

    def test_get_by_tenant(self):
        page_mock = mock.Mock()
        self.datastore_version_members._list = page_mock
        limit = "test-limit"
        marker = "test-marker"
        self.datastore_version_members.get_by_tenant("datastore1", "tenant1",
                                                     limit, marker)
        page_mock.assert_called_with("/mgmt/datastores/datastore1/versions/"
                                     "members/tenant1",
                                     "datastore_version_members",
                                     limit, marker)



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from troveclient import base
from troveclient import common


class SecurityGroup(base.Resource):
    """Security Group is a resource used to hold security group information."""

    def __repr__(self):
        return "<SecurityGroup: %s>" % self.name


class SecurityGroups(base.ManagerWithFind):
    """Manage :class:`SecurityGroup` resources."""
    resource_class = SecurityGroup

    def list(self, limit=None, marker=None):
        """Get a list of all security groups.

        :rtype: list of :class:`SecurityGroup`.
        """
        return self._paginated("/security-groups", "security_groups",
                               limit, marker)

    def get(self, security_group):
        """Get a specific security group.

        :rtype: :class:`SecurityGroup`
        """
        return self._get("/security-groups/%s" % base.getid(security_group),
                         "security_group")


class SecurityGroupRule(base.Resource):
    """This resource is used to hold security group rule information."""

    def __repr__(self):
        return ("<SecurityGroupRule: "
                "( Security Group id: %d, "
                "Protocol: %s, From_Port: %d, "
                "To_Port: %d,  CIDR: %s )>"
                % (self.group_id, self.protocol,
                   self.from_port, self.to_port, self.cidr))


class SecurityGroupRules(base.ManagerWithFind):
    """Manage :class:`SecurityGroupRules` resources."""
    resource_class = SecurityGroupRule

    def create(self, group_id, cidr):
        """Create a new security group rule."""
        body = {"security_group_rule": {
            "group_id": group_id,
            "cidr": cidr
        }}
        return self._create("/security-group-rules", body,
                            "security_group_rule", return_raw=True)

    def delete(self, security_group_rule):
        """Delete the specified security group rule.

        :param security_group_rule: The security group rule to delete
        """
        url = "/security-group-rules/%s" % base.getid(security_group_rule)
        resp, body = self.api.client.delete(url)
        common.check_for_exceptions(resp, body, url)

    # Appease the abc gods
    def list(self):
        pass



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
from unittest import mock

from osc_lib import exceptions
from osc_lib import utils

from troveclient import common
from troveclient.osc.v1 import database_users
from troveclient.tests.osc.v1 import fakes


class TestUsers(fakes.TestDatabasev1):
    fake_users = fakes.FakeUsers()

    def setUp(self):
        super(TestUsers, self).setUp()
        self.user_client = self.app.client_manager.database.users


class TestDatabaseUserCreate(TestUsers):

    def setUp(self):
        super(TestDatabaseUserCreate, self).setUp()
        self.cmd = database_users.CreateDatabaseUser(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_user_create(self, mock_find):
        args = ['instance1', 'user1', 'password1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        user = {'name': 'user1', 'password': 'password1', 'databases': []}
        self.user_client.create.assert_called_with('instance1', [user])
        self.assertIsNone(result)

    @mock.patch.object(utils, 'find_resource')
    def test_user_create_with_optional_args(self, mock_find):
        args = ['instance2', 'user2', 'password2',
                '--host', '1.1.1.1',
                '--databases', 'db1', 'db2']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        user = {'name': 'user2', 'password': 'password2',
                'host': '1.1.1.1',
                'databases': [{'name': 'db1'}, {'name': 'db2'}]}
        self.user_client.create.assert_called_with('instance2', [user])
        self.assertIsNone(result)


class TestUserList(TestUsers):
    columns = database_users.ListDatabaseUsers.columns
    values = ('harry', '%', 'db1')

    def setUp(self):
        super(TestUserList, self).setUp()
        self.cmd = database_users.ListDatabaseUsers(self.app, None)
        data = [self.fake_users.get_instances_1234_users_harry()]
        self.user_client.list.return_value = common.Paginated(data)

    @mock.patch.object(utils, 'find_resource')
    def test_user_list_defaults(self, mock_find):
        args = ['my_instance']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.user_client.list.assert_called_once_with(*args)
        self.assertEqual(self.columns, columns)
        self.assertEqual([self.values], data)


class TestUserShow(TestUsers):
    values = ([{'name': 'db1'}], '%', 'harry')

    def setUp(self):
        super(TestUserShow, self).setUp()
        self.cmd = database_users.ShowDatabaseUser(self.app, None)
        self.data = self.fake_users.get_instances_1234_users_harry()
        self.user_client.get.return_value = self.data
        self.columns = (
            'databases',
            'host',
            'name',
        )

    @mock.patch.object(utils, 'find_resource')
    def test_user_show_defaults(self, mock_find):
        args = ['my_instance', 'harry']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)


class TestDatabaseUserDelete(TestUsers):

    def setUp(self):
        super(TestDatabaseUserDelete, self).setUp()
        self.cmd = database_users.DeleteDatabaseUser(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_user_delete(self, mock_find):
        args = ['userinstance', 'user1', '--host', '1.1.1.1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.user_client.delete.assert_called_with('userinstance',
                                                   'user1',
                                                   '1.1.1.1')
        self.assertIsNone(result)

    @mock.patch.object(utils, 'find_resource')
    def test_user_delete_without_host(self, mock_find):
        args = ['userinstance2', 'user1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.user_client.delete.assert_called_with('userinstance2',
                                                   'user1',
                                                   None)
        self.assertIsNone(result)

    @mock.patch.object(utils, 'find_resource')
    def test_user_delete_with_exception(self, mock_find):
        args = ['userfakeinstance', 'db1', '--host', '1.1.1.1']
        parsed_args = self.check_parser(self.cmd, args, [])

        mock_find.side_effect = exceptions.CommandError
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)


class TestDatabaseUserGrantAccess(TestUsers):

    def setUp(self):
        super(TestDatabaseUserGrantAccess, self).setUp()
        self.cmd = database_users.GrantDatabaseUserAccess(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_user_grant_access(self, mock_find):
        args = ['userinstance', 'user1', '--host', '1.1.1.1', 'db1']
        verifylist = [
            ('instance', 'userinstance'),
            ('name', 'user1'),
            ('host', '1.1.1.1'),
            ('databases', ['db1']),
        ]
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.assertIsNone(result)


class TestDatabaseUserRevokeAccess(TestUsers):

    def setUp(self):
        super(TestDatabaseUserRevokeAccess, self).setUp()
        self.cmd = database_users.RevokeDatabaseUserAccess(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_user_grant_access(self, mock_find):
        args = ['userinstance', 'user1', '--host', '1.1.1.1', 'db1']
        verifylist = [
            ('instance', 'userinstance'),
            ('name', 'user1'),
            ('host', '1.1.1.1'),
            ('databases', 'db1'),
        ]
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.assertIsNone(result)


class TestDatabaseUserShowAccess(TestUsers):
    columns = database_users.ShowDatabaseUserAccess.columns
    values = [('db_1',), ('db_2',)]

    def setUp(self):
        super(TestDatabaseUserShowAccess, self).setUp()
        self.cmd = database_users.ShowDatabaseUserAccess(self.app, None)
        self.data = self.fake_users.get_instances_1234_users_access()
        self.user_client.list_access.return_value = self.data

    @mock.patch.object(utils, 'find_resource')
    def test_user_grant_access(self, mock_find):
        args = ['userinstance', 'user1', '--host', '1.1.1.1']
        verifylist = [
            ('instance', 'userinstance'),
            ('name', 'user1'),
            ('host', '1.1.1.1'),
        ]
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)


class TestDatabaseUserUpdateAttributes(TestUsers):

    def setUp(self):
        super(TestDatabaseUserUpdateAttributes, self).setUp()
        self.cmd = database_users.UpdateDatabaseUserAttributes(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_user__update_attributes(self, mock_find):
        args = ['userinstance',
                'user1',
                '--host', '1.1.1.1',
                '--new_name', 'user2',
                '--new_password', '111111',
                '--new_host', '1.1.1.2']
        verifylist = [
            ('instance', 'userinstance'),
            ('name', 'user1'),
            ('host', '1.1.1.1'),
            ('new_name', 'user2'),
            ('new_password', '111111'),
            ('new_host', '1.1.1.2'),
        ]
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.assertIsNone(result)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Datastores action implementations"""

from osc_lib.command import command
from osc_lib import utils

from troveclient import exceptions
from troveclient.i18n import _
from troveclient import utils as tc_utils


def set_attributes_for_print_detail(datastore):
    info = datastore._info.copy()
    versions = info.get('versions', [])
    versions_str = "\n".join(
        [ver['name'] + " (" + ver['id'] + ")" for ver in versions])
    info['versions (id)'] = versions_str
    info.pop('versions', None)
    info.pop('links', None)
    if hasattr(datastore, 'default_version'):
        def_ver_id = getattr(datastore, 'default_version')
        info['default_version'] = [
            ver['name'] for ver in versions if ver['id'] == def_ver_id][0]
    return info


class ListDatastores(command.Lister):

    _description = _("List available datastores")
    columns = ['ID', 'Name']

    def take_action(self, parsed_args):
        datastore_client = self.app.client_manager.database.datastores
        datastores = datastore_client.list()
        ds = [utils.get_item_properties(d, self.columns) for d in datastores]
        return self.columns, ds


class ShowDatastore(command.ShowOne):
    _description = _("Shows details of a datastore")

    def get_parser(self, prog_name):
        parser = super(ShowDatastore, self).get_parser(prog_name)
        parser.add_argument(
            'datastore',
            metavar='<datastore>',
            help=_('ID of the datastore'),
        )
        return parser

    def take_action(self, parsed_args):
        datastore_client = self.app.client_manager.database.datastores
        datastore = utils.find_resource(datastore_client,
                                        parsed_args.datastore)
        datastore = set_attributes_for_print_detail(datastore)
        return zip(*sorted(datastore.items()))


class DeleteDatastore(command.Command):
    _description = _("Deletes a datastore")

    def get_parser(self, prog_name):
        parser = super(DeleteDatastore, self).get_parser(prog_name)
        parser.add_argument(
            'datastore',
            metavar='<datastore>',
            help=_('ID or name of the datastore'),
        )
        return parser

    def take_action(self, parsed_args):
        datastore_client = self.app.client_manager.database.datastores
        try:
            datastore_client.delete(parsed_args.datastore)
        except Exception as e:
            msg = (_("Failed to delete datastore %(datastore)s: %(e)s")
                   % {'datastore': parsed_args.datastore, 'e': e})
            raise exceptions.CommandError(msg)


class ListDatastoreVersions(command.Lister):

    _description = _("Lists available versions for a datastore")
    columns = ['ID', 'Name', "Version"]

    def get_parser(self, prog_name):
        parser = super(ListDatastoreVersions, self).get_parser(prog_name)
        parser.add_argument(
            'datastore',
            metavar='<datastore>',
            help=_('ID or name of the datastore'),
        )
        return parser

    def take_action(self, parsed_args):
        datastore_version_client = \
            self.app.client_manager.database.datastore_versions
        versions = datastore_version_client.list(parsed_args.datastore)
        ds = [utils.get_dict_properties(d.to_dict(), self.columns)
              for d in versions]
        return self.columns, ds


class ShowDatastoreVersion(command.ShowOne):
    _description = _("Shows details of a datastore version.")

    def get_parser(self, prog_name):
        parser = super(ShowDatastoreVersion, self).get_parser(prog_name)
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('ID or name of the datastore version.'),
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_('ID or name of the datastore. Optional if the ID of'
                   'the datastore_version is provided.'),
        )
        return parser

    def take_action(self, parsed_args):
        datastore_version_client =\
            self.app.client_manager.database.datastore_versions
        if parsed_args.datastore:
            datastore_version = datastore_version_client.\
                get(parsed_args.datastore, parsed_args.datastore_version)
        elif tc_utils.is_uuid_like(parsed_args.datastore_version):
            datastore_version = datastore_version_client.\
                get_by_uuid(parsed_args.datastore_version)
        else:
            raise exceptions.NoUniqueMatch(_('The datastore name or id is'
                                             ' required to retrieve a'
                                             ' datastore version by name.'))
        if datastore_version._info.get('links'):
            del (datastore_version._info['links'])
        return zip(*sorted(datastore_version._info.items()))


class DeleteDatastoreVersion(command.Command):
    _description = _("Deletes a datastore version.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatastoreVersion, self).get_parser(prog_name)
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('ID of the datastore version.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.database.mgmt_ds_versions
        try:
            client.delete(parsed_args.datastore_version)
        except Exception as e:
            msg = (_("Failed to delete datastore version %(version)s: %(e)s")
                   % {'version': parsed_args.datastore_version, 'e': e})
            raise exceptions.CommandError(msg)


class CreateDatastoreVersion(command.Command):
    _description = _("Creates a datastore version.")

    def get_parser(self, prog_name):
        parser = super(CreateDatastoreVersion, self).get_parser(prog_name)
        parser.add_argument(
            'version_name',
            help=_('Datastore version name.'),
        )
        parser.add_argument(
            'datastore_name',
            help=_("Datastore name. The datastore is created automatically "
                   "if does not exist."),
        )
        parser.add_argument(
            'datastore_manager',
            help=_('Datastore manager, e.g. mysql'),
        )
        parser.add_argument(
            'image_id',
            help=_('ID of the datastore image in Glance. This can be empty '
                   'string if --image-tags is specified.'),
        )
        parser.add_argument(
            '--active',
            action='store_true',
            help=_('Enable the datastore version.'),
        )
        parser.add_argument(
            '--image-tags',
            help=_('List of image tags separated by comma, e.g. trove,mysql'),
        )
        parser.add_argument(
            '--default',
            action='store_true',
            help=_('If set the datastore version as default.'),
        )
        parser.add_argument(
            '--version-number',
            help=_("The version number for the database. If not specified, "
                   "use the version name as the default value."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.database.mgmt_ds_versions

        image_tags = []
        if parsed_args.image_tags:
            image_tags = parsed_args.image_tags.split(',')

        try:
            client.create(
                parsed_args.version_name,
                parsed_args.datastore_name,
                parsed_args.datastore_manager,
                parsed_args.image_id,
                image_tags=image_tags,
                active='true' if parsed_args.active else 'false',
                default='true' if parsed_args.default else 'false',
                version=parsed_args.version_number
            )
        except Exception as e:
            msg = (_("Failed to create datastore version %(version)s: %(e)s")
                   % {'version': parsed_args.version_name, 'e': e})
            raise exceptions.CommandError(msg)


class UpdateDatastoreVersion(command.Command):
    _description = _("Updates a datastore version.")

    def get_parser(self, prog_name):
        parser = super(UpdateDatastoreVersion, self).get_parser(prog_name)
        parser.add_argument(
            'datastore_version_id',
            help=_('Datastore version ID.'),
        )
        parser.add_argument(
            '--datastore-manager',
            default=None,
            help=_("Datastore manager name."),
        )
        parser.add_argument(
            '--image',
            default=None,
            help=_('ID of the datastore image in Glance.'),
        )
        parser.add_argument(
            '--image-tags',
            default=None,
            help=_('List of image tags separated by comma, e.g. trove,mysql'),
        )
        parser.add_argument(
            '--version-name',
            help=_('New datastore version name.'),
        )

        enable_group = parser.add_mutually_exclusive_group()
        enable_group.add_argument('--enable', dest='enable',
                                  default=None,
                                  action='store_const',
                                  const='true')
        enable_group.add_argument('--disable', dest='enable',
                                  default=None,
                                  action='store_const',
                                  const='false')

        default_group = parser.add_mutually_exclusive_group()
        default_group.add_argument('--default', dest='default',
                                   default=None,
                                   action='store_const',
                                   const='true')
        default_group.add_argument('--non-default', dest='default',
                                   default=None,
                                   action='store_const',
                                   const='false')

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.database.mgmt_ds_versions

        image_tags = None
        if parsed_args.image_tags is not None:
            image_tags = parsed_args.image_tags.split(',')

        try:
            client.edit(
                parsed_args.datastore_version_id,
                datastore_manager=parsed_args.datastore_manager,
                image=parsed_args.image,
                image_tags=image_tags,
                active=parsed_args.enable, default=parsed_args.default,
                name=parsed_args.version_name
            )
        except Exception as e:
            msg = (_("Failed to update datastore version %(version)s: %(e)s")
                   % {'version': parsed_args.datastore_version_id, 'e': e})
            raise exceptions.CommandError(msg)



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import fixtures
from keystoneauth1 import adapter
import logging
import requests
import testtools

from troveclient.apiclient import client
from troveclient import client as other_client
from troveclient import exceptions
from troveclient import service_catalog
import troveclient.v1.client


class ClientTest(testtools.TestCase):

    def test_get_client_class_v1(self):
        version_map = other_client.get_version_map()
        output = client.BaseClient.get_class('database',
                                             '1.0', version_map)
        self.assertEqual(troveclient.v1.client.Client, output)

    def test_get_client_class_unknown(self):
        version_map = other_client.get_version_map()
        self.assertRaises(exceptions.UnsupportedVersion,
                          client.BaseClient.get_class, 'database',
                          '0', version_map)

    def test_client_with_auth_system_without_auth_plugin(self):
        self.assertRaisesRegex(
            exceptions.AuthSystemNotFound, "AuthSystemNotFound: 'something'",
            other_client.HTTPClient, user='user', password='password',
            projectid='project', timeout=2, auth_url="http://www.blah.com",
            auth_system='something')

    def test_client_with_auth_system_without_endpoint(self):
        auth_plugin = mock.Mock()
        auth_plugin.get_auth_url = mock.Mock(return_value=None)
        self.assertRaises(
            exceptions.EndpointNotFound,
            other_client.HTTPClient, user='user', password='password',
            projectid='project', timeout=2, auth_plugin=auth_plugin,
            auth_url=None, auth_system='something')

    def test_client_with_timeout(self):
        instance = other_client.HTTPClient(user='user',
                                           password='password',
                                           projectid='project',
                                           timeout=2,
                                           auth_url="http://www.blah.com",
                                           insecure=True)
        self.assertEqual(2, instance.timeout)
        mock_request = mock.Mock()
        mock_request.return_value = requests.Response()
        mock_request.return_value.status_code = 200
        mock_request.return_value.headers = {
            'x-server-management-url': 'blah.com',
            'x-auth-token': 'blah',
        }
        with mock.patch('requests.request', mock_request):
            instance.authenticate()
            requests.request.assert_called_with(
                mock.ANY, mock.ANY, timeout=2, headers=mock.ANY,
                verify=mock.ANY)

    def test_client_unauthorized(self):
        instance = other_client.HTTPClient(user='user',
                                           password='password',
                                           projectid='project',
                                           timeout=2,
                                           auth_url="http://www.blah.com",
                                           cacert=mock.Mock())
        instance.auth_token = 'foobar'
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v2.0'
        mock_request = mock.Mock()
        mock_request.side_effect = other_client.exceptions.Unauthorized(401)
        with mock.patch('requests.request', mock_request):
            self.assertRaises(
                exceptions.Unauthorized, instance.get, '/instances')

    def test_client_bad_request(self):
        instance = other_client.HTTPClient(user='user',
                                           password='password',
                                           projectid='project',
                                           timeout=2,
                                           auth_url="http://www.blah.com")
        instance.auth_token = 'foobar'
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v2.0'
        mock_request = mock.Mock()
        mock_request.side_effect = other_client.exceptions.BadRequest()
        with mock.patch('requests.request', mock_request):
            self.assertRaises(
                exceptions.BadRequest, instance.get, '/instances')

    def test_client_with_client_exception(self):
        instance = other_client.HTTPClient(user='user',
                                           password='password',
                                           projectid='project',
                                           timeout=2,
                                           auth_url="http://www.blah.com",
                                           retries=2)
        instance.auth_token = 'foobar'
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v2.0'
        mock_request = mock.Mock()
        mock_request.side_effect = other_client.exceptions.ClientException()
        type(mock_request.side_effect).code = mock.PropertyMock(
            side_effect=[501, 111])
        with mock.patch('requests.request', mock_request):
            self.assertRaises(
                exceptions.ClientException, instance.get, '/instances')

    def test_client_connection_error(self):
        instance = other_client.HTTPClient(user='user',
                                           password='password',
                                           projectid='project',
                                           timeout=2,
                                           auth_url="http://www.blah.com",
                                           retries=2)
        instance.auth_token = 'foobar'
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v2.0'
        mock_request = mock.Mock()
        mock_request.side_effect = requests.exceptions.ConnectionError(
            'connection refused')
        with mock.patch('requests.request', mock_request):
            self.assertRaisesRegex(
                exceptions.ClientException,
                'Unable to establish connection: connection refused',
                instance.get, '/instances')

    @mock.patch.object(other_client.HTTPClient, 'request',
                       return_value=(200, "{'versions':[]}"))
    def _check_version_url(self, management_url, version_url, mock_request):
        projectid = '25e469aa1848471b875e68cde6531bc5'
        instance = other_client.HTTPClient(user='user',
                                           password='password',
                                           projectid=projectid,
                                           auth_url="http://www.blah.com")
        instance.auth_token = 'foobar'
        instance.management_url = management_url % projectid
        mock_get_service_url = mock.Mock(return_value=instance.management_url)
        instance.get_service_url = mock_get_service_url
        instance.version = 'v2.0'

        # If passing None as the part of url, a client accesses the url which
        # doesn't include "v2/<projectid>" for getting API version info.
        instance.get('')
        mock_request.assert_called_once_with(instance.management_url, 'GET',
                                             headers=mock.ANY)
        mock_request.reset_mock()

        # Otherwise, a client accesses the url which includes "v2/<projectid>".
        instance.get('/instances')
        url = instance.management_url + '/instances'
        mock_request.assert_called_once_with(url, 'GET', headers=mock.ANY)

    def test_client_version_url(self):
        self._check_version_url('http://foo.com/v1/%s', 'http://foo.com/')

    def test_client_version_url_with_tenant_name(self):
        self._check_version_url('http://foo.com/trove/v1/%s',
                                'http://foo.com/trove/')

    def test_log_req(self):
        logger = self.useFixture(
            fixtures.FakeLogger(
                name='troveclient.client',
                format="%(message)s",
                level=logging.DEBUG,
                nuke_handlers=True
            )
        )
        cs = other_client.HTTPClient(user='user',
                                     password='password',
                                     projectid=None,
                                     auth_url="http://www.blah.com",
                                     http_log_debug=True)
        cs.http_log_req(('/foo', 'GET'), {'headers': {}})
        cs.http_log_req(('/foo', 'GET'),
                        {'headers': {'X-Auth-Token': 'totally_bogus'}})
        cs.http_log_req(
            ('/foo', 'GET'),
            {'headers': {},
             'data': '{"auth": {"passwordCredentials": '
             '{"password": "password"}}}'})

        output = logger.output.split('\n')

        self.assertIn("REQ: curl -i /foo -X GET", output)
        self.assertIn(
            "REQ: curl -i /foo -X GET -H "
            '"X-Auth-Token: totally_bogus"',
            output)
        self.assertIn(
            "REQ: curl -i /foo -X GET -d "
            '\'{"auth": {"passwordCredentials": {"password":'
            ' "password"}}}\'',
            output)

    @mock.patch.object(service_catalog, 'ServiceCatalog')
    def test_client_auth_token(self, mock_service_catalog):
        auth_url = 'http://www.blah.com'
        proxy_token = 'foobar'
        proxy_tenant_id = 'user'
        mock_service_catalog.return_value.get_token = mock.Mock(
            return_value=proxy_token)
        instance = other_client.HTTPClient(proxy_token=proxy_token,
                                           proxy_tenant_id=proxy_tenant_id,
                                           user=None,
                                           password=None,
                                           tenant_id=proxy_tenant_id,
                                           projectid=None,
                                           timeout=2,
                                           auth_url=auth_url)
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v2.0'
        mock_request = mock.Mock()
        mock_request.return_value = requests.Response()
        mock_request.return_value.status_code = 200
        mock_request.return_value.headers = {
            'x-server-management-url': 'blah.com',
            'x-auth-token': 'blah',
        }

        with mock.patch('requests.request', mock_request):
            instance.authenticate()
            mock_request.assert_called_with(
                'GET', auth_url + '/tokens/foobar?belongsTo=user',
                headers={'User-Agent': 'python-troveclient',
                         'Accept': 'application/json',
                         'X-Auth-Token': proxy_token},
                timeout=2, verify=True)

    @mock.patch.object(service_catalog, 'ServiceCatalog', side_effect=KeyError)
    def test_client_auth_token_authorization_failure(self,
                                                     mock_service_catalog):
        auth_url = 'http://www.blah.com'
        proxy_token = 'foobar'
        proxy_tenant_id = 'user'
        mock_service_catalog.return_value.get_token = mock.Mock(
            return_value=proxy_token)
        instance = other_client.HTTPClient(proxy_token=proxy_token,
                                           proxy_tenant_id=proxy_tenant_id,
                                           user=None,
                                           password=None,
                                           tenant_id=proxy_tenant_id,
                                           projectid=None,
                                           timeout=2,
                                           auth_url=auth_url)
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v2.0'
        mock_request = mock.Mock()
        mock_request.return_value = requests.Response()
        mock_request.return_value.status_code = 200
        mock_request.return_value.headers = {
            'x-server-management-url': 'blah.com',
            'x-auth-token': 'blah',
        }

        with mock.patch('requests.request', mock_request):
            self.assertRaises(exceptions.AuthorizationFailure,
                              instance.authenticate)

    @mock.patch.object(service_catalog, 'ServiceCatalog',
                       side_effect=other_client.exceptions.EndpointNotFound)
    def test_client_auth_token_endpoint_not_found(self, mock_service_catalog):
        auth_url = 'http://www.blah.com'
        proxy_token = 'foobar'
        proxy_tenant_id = 'user'
        mock_service_catalog.return_value.get_token = mock.Mock(
            return_value=proxy_token)
        instance = other_client.HTTPClient(proxy_token=proxy_token,
                                           proxy_tenant_id=proxy_tenant_id,
                                           user=None,
                                           password=None,
                                           tenant_id=proxy_tenant_id,
                                           projectid=None,
                                           timeout=2,
                                           auth_url=auth_url)
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v2.0'
        mock_request = mock.Mock()
        mock_request.return_value = requests.Response()
        mock_request.return_value.status_code = 200
        mock_request.return_value.headers = {
            'x-server-management-url': 'blah.com',
            'x-auth-token': 'blah',
        }

        with mock.patch('requests.request', mock_request):
            self.assertRaises(exceptions.EndpointNotFound,
                              instance.authenticate)

    @mock.patch.object(service_catalog, 'ServiceCatalog')
    def test_client_auth_token_v1_auth_failure(self, mock_service_catalog):
        auth_url = 'http://www.blah.com'
        proxy_token = 'foobar'
        proxy_tenant_id = 'user'
        mock_service_catalog.return_value.get_token = mock.Mock(
            return_value=proxy_token)
        instance = other_client.HTTPClient(proxy_token=proxy_token,
                                           proxy_tenant_id=proxy_tenant_id,
                                           user=None,
                                           password=None,
                                           tenant_id=proxy_tenant_id,
                                           projectid=None,
                                           timeout=2,
                                           auth_url=auth_url)
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v1.0'
        mock_request = mock.Mock()
        mock_request.return_value = requests.Response()
        mock_request.return_value.status_code = 200
        mock_request.return_value.headers = {
            'x-server-management-url': 'blah.com',
            'x-auth-token': 'blah',
        }

        with mock.patch('requests.request', mock_request):
            self.assertRaises(exceptions.NoTokenLookupException,
                              instance.authenticate)

    @mock.patch.object(service_catalog, 'ServiceCatalog')
    def test_client_auth_token_v1_auth(self, mock_service_catalog):
        auth_url = 'http://www.blah.com'
        proxy_token = 'foobar'
        mock_service_catalog.return_value.get_token = mock.Mock(
            return_value=proxy_token)
        instance = other_client.HTTPClient(user='user',
                                                password='password',
                                                projectid='projectid',
                                                timeout=2,
                                                auth_url=auth_url)
        instance.management_url = 'http://example.com'
        instance.get_service_url = mock.Mock(return_value='http://example.com')
        instance.version = 'v1.0'
        mock_request = mock.Mock()
        mock_request.return_value = requests.Response()
        mock_request.return_value.status_code = 200
        mock_request.return_value.headers = {
            'x-server-management-url': 'blah.com',
        }
        headers = {'Content-Type': 'application/json',
                   'Accept': 'application/json',
                   'User-Agent': 'python-troveclient'}
        with mock.patch('requests.request', mock_request):
            instance.authenticate()
            called_args, called_kwargs = mock_request.call_args
            self.assertEqual(('POST', 'http://www.blah.com/v2.0/tokens'),
                             called_args)
            self.assertEqual(headers, called_kwargs['headers'])

    def test_client_get(self):
        auth_url = 'http://www.blah.com'
        instance = other_client.HTTPClient(user='user',
                                                password='password',
                                                projectid='project_id',
                                                timeout=2,
                                                auth_url=auth_url)
        instance._cs_request = mock.Mock()

        instance.get('clusters')
        instance._cs_request.assert_called_with('clusters', 'GET')

    def test_client_patch(self):
        auth_url = 'http://www.blah.com'
        body = mock.Mock()
        instance = other_client.HTTPClient(user='user',
                                                password='password',
                                                projectid='project_id',
                                                timeout=2,
                                                auth_url=auth_url)
        instance._cs_request = mock.Mock()

        instance.patch('instances/dummy-instance-id', body=body)
        instance._cs_request.assert_called_with(
            'instances/dummy-instance-id', 'PATCH', body=body)

    def test_client_post(self):
        auth_url = 'http://www.blah.com'
        body = {"add_shard": {}}
        instance = other_client.HTTPClient(user='user',
                                                password='password',
                                                projectid='project_id',
                                                timeout=2,
                                                auth_url=auth_url)
        instance._cs_request = mock.Mock()

        instance.post('clusters/dummy-cluster-id', body=body)
        instance._cs_request.assert_called_with(
            'clusters/dummy-cluster-id', 'POST', body=body)

    def test_client_put(self):
        auth_url = 'http://www.blah.com'
        body = {"user": {"password": "new_password"}}
        instance = other_client.HTTPClient(user='user',
                                                password='password',
                                                projectid='project_id',
                                                timeout=2,
                                                auth_url=auth_url)
        instance._cs_request = mock.Mock()

        instance.put('instances/dummy-instance-id/user/dummy-user', body=body)
        instance._cs_request.assert_called_with(
            'instances/dummy-instance-id/user/dummy-user', 'PUT', body=body)

    def test_client_delete(self):
        auth_url = 'http://www.blah.com'
        instance = other_client.HTTPClient(user='user',
                                                password='password',
                                                projectid='project_id',
                                                timeout=2,
                                                auth_url=auth_url)
        instance._cs_request = mock.Mock()

        instance.delete('/backups/dummy-backup-id')
        instance._cs_request.assert_called_with('/backups/dummy-backup-id',
                                                'DELETE')

    @mock.patch.object(adapter.LegacyJsonAdapter, 'request')
    def test_database_service_name(self, m_request):
        m_request.return_value = (mock.MagicMock(status_code=200), None)

        client = other_client.SessionClient(session=mock.MagicMock(),
                                            auth=mock.MagicMock())
        client.request("http://no.where", 'GET')
        self.assertIsNone(client.database_service_name)

        client = other_client.SessionClient(session=mock.MagicMock(),
                                            auth=mock.MagicMock(),
                                            database_service_name='myservice')
        client.request("http://no.where", 'GET')
        self.assertEqual('myservice', client.database_service_name)

    @mock.patch.object(adapter.LegacyJsonAdapter, 'request')
    @mock.patch.object(adapter.LegacyJsonAdapter, 'get_endpoint',
                       return_value=None)
    def test_error_sessionclient(self, m_end_point, m_request):
        m_request.return_value = (mock.MagicMock(status_code=200), None)

        self.assertRaises(exceptions.EndpointNotFound,
                          other_client.SessionClient,
                          session=mock.MagicMock(),
                          auth=mock.MagicMock())

    def test_construct_http_client(self):
        mock_request = mock.Mock()
        mock_request.return_value = requests.Response()
        mock_request.return_value.status_code = 200
        mock_request.return_value.headers = {
            'x-server-management-url': 'blah.com',
            'x-auth-token': 'blah',
        }
        with mock.patch('requests.request', mock_request):
            self.assertIsInstance(other_client._construct_http_client(),
                                  other_client.HTTPClient)
            self.assertIsInstance(
                other_client._construct_http_client(session=mock.Mock(),
                                                    auth=mock.Mock()),
                other_client.SessionClient)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Flavors action implementations"""

from osc_lib.command import command
from osc_lib import utils
import six

from troveclient import exceptions
from troveclient.i18n import _


def set_attributes_for_print_detail(flavor):
    info = flavor._info.copy()
    # Get rid of those ugly links
    if info.get('links'):
        del(info['links'])

    # Fallback to str_id for flavors, where necessary
    if hasattr(flavor, 'str_id'):
        info['id'] = flavor.id
        del(info['str_id'])
    return info


class ListDatabaseFlavors(command.Lister):

    _description = _("List database flavors")
    columns = ['ID', 'Name', 'RAM', 'vCPUs', 'Disk', 'Ephemeral']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseFlavors, self).get_parser(prog_name)
        parser.add_argument(
            '--datastore-type',
            dest='datastore_type',
            metavar='<datastore-type>',
            help=_('Type of the datastore. For eg: mysql.')
        )
        parser.add_argument(
            '--datastore-version-id',
            dest='datastore_version_id',
            metavar='<datastore-version-id>',
            help=_('ID of the datastore version.')
        )
        return parser

    def take_action(self, parsed_args):
        db_flavors = self.app.client_manager.database.flavors
        if parsed_args.datastore_type and parsed_args.datastore_version_id:
            flavors = db_flavors.list_datastore_version_associated_flavors(
                datastore=parsed_args.datastore_type,
                version_id=parsed_args.datastore_version_id)
        elif (not parsed_args.datastore_type and not
              parsed_args.datastore_version_id):
            flavors = db_flavors.list()
        else:
            raise exceptions.MissingArgs(['datastore-type',
                                          'datastore-version-id'])

        # Fallback to str_id where necessary.
        _flavors = []
        for f in flavors:
            if not f.id and hasattr(f, 'str_id'):
                f.id = f.str_id
            _flavors.append(utils.get_item_properties(f, self.columns))

        return self.columns, _flavors


class ShowDatabaseFlavor(command.ShowOne):
    _description = _("Shows details of a database flavor")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar='<flavor>',
            help=_('ID or name of the flavor'),
        )
        return parser

    def take_action(self, parsed_args):
        db_flavors = self.app.client_manager.database.flavors
        flavor = utils.find_resource(db_flavors,
                                     parsed_args.flavor)
        flavor = set_attributes_for_print_detail(flavor)
        return zip(*sorted(six.iteritems(flavor)))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock

from troveclient.tests import fakes
from troveclient.tests.osc import utils
from troveclient.v1 import backups
from troveclient.v1 import clusters
from troveclient.v1 import configurations
from troveclient.v1 import databases
from troveclient.v1 import datastores
from troveclient.v1 import flavors
from troveclient.v1 import instances
from troveclient.v1 import limits
from troveclient.v1 import modules
from troveclient.v1 import quota
from troveclient.v1 import users


class TestDatabasev1(utils.TestCommand):
    def setUp(self):
        super(TestDatabasev1, self).setUp()
        self.app.client_manager.database = mock.MagicMock()


class FakeFlavors(object):
    fake_flavors = fakes.FakeHTTPClient().get_flavors()[2]['flavors']

    def get_flavors_1(self):
        return flavors.Flavor(None, self.fake_flavors[0])


class FakeBackups(object):
    fake_backups = fakes.FakeHTTPClient().get_backups()[2]['backups']

    def get_backup_bk_1234(self):
        return backups.Backup(None, self.fake_backups[0])


class FakeClusters(object):
    fake_clusters = fakes.FakeHTTPClient().get_clusters()[2]['clusters']
    fake_cluster = (fakes.FakeHTTPClient()
                    .get_clusters_cls_1234()[2]['cluster'])
    fake_cluster_member = fake_cluster['instances'][1]
    fake_cluster_instance_modules = (fakes.FakeHTTPClient().
                                     get_cluster_instance_modules()[2]
                                     ['modules'])

    def get_clusters_cls_1234(self):
        return clusters.Cluster(None, self.fake_cluster)

    def get_clusters_member_2(self):
        return instances.Instance(None, self.fake_cluster_member)

    def cluster_instance_modules(self):
        return [[modules.Module(None, mod)]
                for mod in self.fake_cluster_instance_modules]


class FakeConfigurations(object):
    fake_config = (fakes.FakeHTTPClient().get_configurations()
                   [2]['configurations'])
    fake_config_instances = (fakes.FakeHTTPClient().
                             get_configurations_c_123_instances()[2])
    fake_default_config = (
        fakes.FakeHTTPClient().get_instances_1234_configuration()
        [2]['instance'])

    def get_configurations_c_123(self):
        return configurations.Configuration(None, self.fake_config[0])

    def get_configuration_instances(self):
        return [instances.Instance(None, fake_instance)
                for fake_instance in self.fake_config_instances['instances']]

    def get_default_configuration(self):
        return instances.Instance(None, self.fake_default_config)


class FakeConfigurationParameters(object):
    fake_config_param = (fakes.FakeHTTPClient().
                         get_datastores_d_123_versions_v_156_parameters()
                         [2]['configuration-parameters'])

    def get_params_connect_timeout(self):
        return configurations.\
            ConfigurationParameter(None, self.fake_config_param[1])


class FakeLimits(object):
    fake_limits = fakes.FakeHTTPClient().get_limits()[2]['limits']

    def get_absolute_limits(self):
        return limits.Limit(None, self.fake_limits[0])

    def get_non_absolute_limits(self):
        return limits.Limit(None,
                            {'value': 200,
                             'verb': 'DELETE',
                             'remaining': 200,
                             'unit': 'MINUTE'})


class FakeUsers(object):
    fake_users = fakes.FakeHTTPClient().get_instances_1234_users()[2]['users']
    fake_user_access = fakes.FakeHTTPClient().\
        get_instances_1234_users_jacob_databases()[2]

    def get_instances_1234_users_harry(self):
        return users.User(None, self.fake_users[2])

    def get_instances_1234_users_access(self):
        return [databases.Database(self, db) for db in
                self.fake_user_access['databases']]


class FakeInstances(object):
    fake_instances = (fakes.FakeHTTPClient().get_instances()[2]['instances'])
    fake_instance = fakes.FakeHTTPClient().get_instance_create()[2]

    def get_instances_1234(self):
        return instances.Instance(None, self.fake_instances[0])

    def get_instances(self):
        return [instances.Instance(None, fake_instance)
                for fake_instance in self.fake_instances]

    def get_instance_create(self):
        return instances.Instance(None, self.fake_instance['instance'])


class FakeDatabases(object):
    fake_databases = [{'name': 'fakedb1'}]

    def get_databases_1(self):
        return databases.Database(None, self.fake_databases[0])


class FakeDatastores(object):
    fake_datastores = fakes.FakeHTTPClient().get_datastores()[2]['datastores']
    fake_datastore_versions = fake_datastores[0]['versions']

    def get_datastores_d_123(self):
        return datastores.Datastore(None, self.fake_datastores[0])

    def get_datastores_d_123_versions(self):
        return datastores.Datastore(None, self.fake_datastore_versions[0])


class FakeRoot(object):
    fake_instance_1234_root = (fakes.FakeHTTPClient()
                               .get_instances_1234_root()[2])
    fake_cls_1234_root = (fakes.FakeHTTPClient()
                          .get_clusters_cls_1234_root()[2])

    def get_instance_1234_root(self):
        return users.User(None, self.fake_instance_1234_root,
                          loaded=True)

    def get_cls_1234_root(self):
        return users.User(None, self.fake_cls_1234_root,
                          loaded=True)

    def post_instance_1234_root(self):
        root = fakes.FakeHTTPClient().post_instances_1234_root()[2]['user']
        return root['name'], root['password']

    def post_cls_1234_root(self):
        root = fakes.FakeHTTPClient().post_instances_1234_root()[2]['user']
        return root['name'], root['password']

    def delete_instance_1234_root(self):
        return fakes.FakeHTTPClient().delete_instances_1234_root()[2]


class FakeQuota(object):
    fake_quotas = fakes.FakeHTTPClient().get_quotas()[2]['quotas']
    fake_instances_quota = (fakes.FakeHTTPClient()
                            .update_instances_quota()[2]['quotas'])

    def get_quotas(self):
        return [quota.Quotas.resource_class(None, q)
                for q in self.fake_quotas]


class FakeLogs(object):
    fake_logs = fakes.FakeHTTPClient().get_logs()[2]['logs']

    def get_logs(self):
        return [instances.DatastoreLog(None, fake_log)
                for fake_log in self.fake_logs]
